@@ -324,6 +324,22 @@ def require_version(modulename, version):
         exit(1)
 
 
+def get_square_width(video_info):
+    """Applies the pixel aspect ratio to the width of the video info.
+
+    Args:
+        video_info (GstPbutils.DiscovererVideoInfo): The video info.
+
+    Returns:
+        int: The width calculated exactly as GStreamer does.
+    """
+    width = video_info.get_width()
+    par_num = video_info.get_par_num()
+    par_denom = video_info.get_par_denom()
+    # We observed GStreamer does a simple int(), so we leave it like this.
+    return int(width * par_num / par_denom)
+
+
 def initialize_modules():
     """Initializes the modules.
 
@@ -341,6 +357,11 @@ def initialize_modules():
     require_version("Gdk", GTK_API_VERSION)
     from gi.repository import Gdk
     Gdk.init([])
+    from gi.repository import Gtk
+
+    # Monkey patch deprecated methods to use the new variant by default
+    Gtk.Layout.get_vadjustment = Gtk.Scrollable.get_vadjustment
+    Gtk.Layout.get_hadjustment = Gtk.Scrollable.get_hadjustment
 
     if not gi.version_info >= (3, 11):
         from gi.repository import GObject
@@ -353,12 +374,21 @@ def initialize_modules():
     from pitivi.configure import get_audiopresets_dir, get_videopresets_dir
     Gst.init(None)
 
+    require_version("GstPbutils", GST_API_VERSION)
+    from gi.repository import GstPbutils
+
+    # Monky patch a helper method for retrieving the size of a video
+    # when using square pixels.
+    GstPbutils.DiscovererVideoInfo.get_square_width = get_square_width
+
     if not os.environ.get("GES_DISCOVERY_TIMEOUT"):
         os.environ["GES_DISCOVERY_TIMEOUT"] = "5"
 
     require_version("GES", GST_API_VERSION)
     from gi.repository import GES
     res, sys.argv = GES.init_check(sys.argv)
+    # Monkey patch deprecated methods to use the new variant by default
+    GES.TrackElement.list_children_properties = GES.TimelineElement.list_children_properties
 
     from pitivi.utils import validate
     if validate.init() and "--inspect-action-type" in sys.argv:
@@ -387,7 +417,7 @@ def initialize_modules():
 # a specific version requirement, they have the "None" value.
 
 GST_API_VERSION = "1.0"
-GST_VERSION = "1.12.3"
+GST_VERSION = "1.14.2"
 GTK_API_VERSION = "3.0"
 GLIB_API_VERSION = "2.0"
 HARD_DEPENDENCIES = [GICheck("3.20.0"),
@@ -399,8 +429,8 @@ HARD_DEPENDENCIES = [GICheck("3.20.0"),
                      GtkDependency("Gtk", GTK_API_VERSION, "3.20.0"),
                      ClassicDependency("numpy"),
                      GIDependency("Gio", "2.0"),
-                     GstPluginDependency("gtk", GST_VERSION),
-                     GstPluginDependency("gdkpixbuf", GST_VERSION),
+                     GstPluginDependency("gtk"),
+                     GstPluginDependency("gdkpixbuf"),
                      ClassicDependency("matplotlib"),
                      ]
 
