@@ -34,6 +34,7 @@ from gettext import gettext as _
 
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
+from gi.repository import GES
 from gi.repository import GLib
 from gi.repository import Gst
 from gi.repository import Gtk
@@ -191,8 +192,25 @@ class EffectInfo(object):
         else:
             return bin_description
 
+    def good_for_track_element(self, track_element):
+        """Checks the effect is compatible with the specified track element.
 
-class EffectsManager(object):
+        Args:
+            track_element (GES.TrackElement): The track element to check against.
+
+        Returns:
+            bool: Whether it makes sense to apply the effect to the track element.
+        """
+        track_type = track_element.get_track_type()
+        if track_type == GES.TrackType.AUDIO:
+            return self.media_type == AUDIO_EFFECT
+        elif track_type == GES.TrackType.VIDEO:
+            return self.media_type == VIDEO_EFFECT
+        else:
+            return False
+
+
+class EffectsManager(Loggable):
     """Keeps info about effects and their categories.
 
     Attributes:
@@ -201,16 +219,18 @@ class EffectsManager(object):
     """
 
     def __init__(self):
-        object.__init__(self)
+        Loggable.__init__(self)
         self.video_effects = []
         self.audio_effects = []
+        self.gl_effects = []
         self._effects = {}
 
         useless_words = ["Video", "Audio", "audio", "effect",
                          _("Video"), _("Audio"), _("Audio").lower(), _("effect")]
         uselessRe = re.compile(" |".join(useless_words))
 
-        factories = Gst.Registry.get().get_feature_list(Gst.ElementFactory)
+        registry = Gst.Registry.get()
+        factories = registry.get_feature_list(Gst.ElementFactory)
         longnames = set()
         duplicate_longnames = set()
         for factory in factories:
@@ -250,6 +270,36 @@ class EffectsManager(object):
                                 description=factory.get_description())
             self._effects[name] = effect
 
+        gl_element_factories = registry.get_feature_list_by_plugin("opengl")
+        self.gl_effects = [element_factory.get_name()
+                           for element_factory in gl_element_factories]
+        if self.gl_effects:
+            # Checking whether the GL effects can be used
+            # by setting a pipeline with "gleffects" to PAUSED.
+            pipeline = Gst.parse_launch("videotestsrc ! glupload ! gleffects ! fakesink")
+            bus = pipeline.get_bus()
+            bus.add_signal_watch()
+            bus.connect("message", self._gl_pipeline_message_cb, pipeline)
+            assert pipeline.set_state(Gst.State.PAUSED) == Gst.StateChangeReturn.ASYNC
+
+    def _gl_pipeline_message_cb(self, bus, message, pipeline):
+        """Handles a `message` event on the pipeline for checking gl effects."""
+        done = False
+        if message.type == Gst.MessageType.ASYNC_DONE:
+            self.debug("GL effects check pipeline successfully PAUSED")
+            done = True
+        elif message.type == Gst.MessageType.ERROR:
+            # The pipeline cannot be set to PAUSED.
+            error, detail = message.parse_error()
+            self.debug("Hiding the GL effects because: %s, %s", error, detail)
+            HIDDEN_EFFECTS.extend(self.gl_effects)
+            done = True
+
+        if done:
+            bus.remove_signal_watch()
+            bus.disconnect_by_func(self._gl_pipeline_message_cb)
+            pipeline.set_state(Gst.State.NULL)
+
     def getInfo(self, bin_description):
         """Gets the info for an effect which can be applied.
 
@@ -483,10 +533,21 @@ class EffectListWidget(Gtk.Box, Loggable):
         return True
 
     def _addSelectedEffect(self):
+        """Adds the selected effect to the single selected clip, if any."""
         effect = self.getSelectedEffect()
-        if not effect:
+        effect_info = self.app.effects.getInfo(effect)
+        if not effect_info:
             return
-        self.app.gui.clipconfig.effect_expander.addEffectToCurrentSelection(effect)
+        timeline = self.app.gui.timeline_ui.timeline
+        clip = timeline.selection.getSingleClip()
+        if not clip:
+            return
+        pipeline = timeline.ges_timeline.get_parent()
+        from pitivi.undo.timeline import CommitTimelineFinalizingAction
+        with self.app.action_log.started("add effect",
+                                         finalizing_action=CommitTimelineFinalizingAction(pipeline),
+                                         toplevel=True):
+            clip.ui.add_effect(effect_info)
 
     def getSelectedEffect(self):
         if self._draggedItems:
@@ -602,6 +663,7 @@ class EffectsPropertiesManager:
 
             pipeline = self.app.project_manager.current_project.pipeline
             with self.app.action_log.started("Effect property change",
-                                             CommitTimelineFinalizingAction(pipeline)):
+                                             finalizing_action=CommitTimelineFinalizingAction(pipeline),
+                                             toplevel=True):
                 effect.set_child_property(prop.name, value)
             self._current_element_values[prop.name] = value
