@@ -18,7 +18,6 @@
 # Boston, MA 02110-1301, USA.
 import os
 from gettext import gettext as _
-from hashlib import md5
 from time import time
 from urllib.parse import unquote
 
@@ -40,6 +39,7 @@ from pitivi.configure import VERSION
 from pitivi.dialogs.prefs import PreferencesDialog
 from pitivi.effects import EffectListWidget
 from pitivi.mediafilespreviewer import PreviewWidget
+from pitivi.medialibrary import AssetThumbnail
 from pitivi.medialibrary import MediaLibraryWidget
 from pitivi.project import ProjectSettingsDialog
 from pitivi.settings import GlobalSettings
@@ -50,10 +50,11 @@ from pitivi.transitions import TransitionsListWidget
 from pitivi.utils.loggable import Loggable
 from pitivi.utils.misc import path_from_uri
 from pitivi.utils.misc import show_user_manual
-from pitivi.utils.ui import beautify_length
+from pitivi.utils.ui import beautify_missing_asset
 from pitivi.utils.ui import beautify_time_delta
 from pitivi.utils.ui import clear_styles
 from pitivi.utils.ui import info_name
+from pitivi.utils.ui import PADDING
 from pitivi.utils.ui import SPACING
 from pitivi.utils.ui import TIMELINE_CSS
 from pitivi.viewer.viewer import ViewerContainer
@@ -104,10 +105,6 @@ GlobalSettings.addConfigOption('lastCurrentVersion',
                                section='version',
                                key='last-current-version',
                                default='')
-GlobalSettings.addConfigOption('timelineAutoRipple',
-                               section='user-interface',
-                               key='timeline-autoripple',
-                               default=False)
 
 
 class MainWindow(Gtk.ApplicationWindow, Loggable):
@@ -222,23 +219,6 @@ class MainWindow(Gtk.ApplicationWindow, Loggable):
         # Main "toolbar" (using client-side window decorations with HeaderBar)
         self._headerbar = Gtk.HeaderBar()
         self._create_headerbar_buttons()
-        self.builder.add_from_file(
-            os.path.join(get_ui_dir(), "mainmenubutton.ui"))
-
-        # FIXME : see https://bugzilla.gnome.org/show_bug.cgi?id=729263
-        self.builder.connect_signals_full(self._builderConnectCb, self)
-
-        self._menubutton = self.builder.get_object("menubutton")
-
-        if Gtk.get_major_version() == 3 and Gtk.get_minor_version() < 13:
-            open_menu_image = self.builder.get_object("open_menu_image")
-            open_menu_image.set_property("icon_name", "emblem-system-symbolic")
-
-        self._menubutton_items = {}
-        for widget in self.builder.get_object("menu").get_children():
-            self._menubutton_items[Gtk.Buildable.get_name(widget)] = widget
-
-        self._headerbar.pack_end(self._menubutton)
         self._headerbar.set_show_close_button(True)
         self._headerbar.show_all()
         self.set_titlebar(self._headerbar)
@@ -258,9 +238,9 @@ class MainWindow(Gtk.ApplicationWindow, Loggable):
         self.main_tabs = BaseTabs(self.app)
         self.medialibrary = MediaLibraryWidget(self.app)
         self.effectlist = EffectListWidget(self.app)
-        self.main_tabs.append_page(
+        self.main_tabs.append_page("Media Library",
             self.medialibrary, Gtk.Label(label=_("Media Library")))
-        self.main_tabs.append_page(
+        self.main_tabs.append_page("Effect Library",
             self.effectlist, Gtk.Label(label=_("Effect Library")))
         self.medialibrary.connect('play', self._mediaLibraryPlayCb)
         self.medialibrary.show()
@@ -271,11 +251,11 @@ class MainWindow(Gtk.ApplicationWindow, Loggable):
         self.clipconfig = ClipProperties(self.app)
         self.trans_list = TransitionsListWidget(self.app)
         self.title_editor = TitleEditor(self.app)
-        self.context_tabs.append_page(
+        self.context_tabs.append_page("Clip",
             self.clipconfig, Gtk.Label(label=_("Clip")))
-        self.context_tabs.append_page(
+        self.context_tabs.append_page("Transition",
             self.trans_list, Gtk.Label(label=_("Transition")))
-        self.context_tabs.append_page(
+        self.context_tabs.append_page("Title",
             self.title_editor.widget, Gtk.Label(label=_("Title")))
         # Show by default the Title tab, as the Clip and Transition tabs
         # are useful only when a clip or transition is selected, but
@@ -396,32 +376,25 @@ class MainWindow(Gtk.ApplicationWindow, Loggable):
             layers_representation.grab_focus()
 
     def _create_headerbar_buttons(self):
+
         undo_button = Gtk.Button.new_from_icon_name(
-            "edit-undo-symbolic", Gtk.IconSize.LARGE_TOOLBAR)
+            "edit-undo-symbolic", Gtk.IconSize.SMALL_TOOLBAR)
         undo_button.set_always_show_image(True)
         undo_button.set_label(_("Undo"))
         undo_button.set_action_name("app.undo")
         undo_button.set_use_underline(True)
 
         redo_button = Gtk.Button.new_from_icon_name(
-            "edit-redo-symbolic", Gtk.IconSize.LARGE_TOOLBAR)
+            "edit-redo-symbolic", Gtk.IconSize.SMALL_TOOLBAR)
         redo_button.set_always_show_image(True)
-        redo_button.set_label(_("Redo"))
         redo_button.set_action_name("app.redo")
         redo_button.set_use_underline(True)
 
-        separator = Gtk.Separator()
-
-        self.save_button = Gtk.Button.new_from_icon_name(
-            "document-save", Gtk.IconSize.LARGE_TOOLBAR)
-        self.save_button.set_always_show_image(True)
-        self.save_button.set_label(_("Save"))
+        self.save_button = Gtk.Button.new_with_label(_("Save"))
         self.save_button.set_focus_on_click(False)
 
-        render_icon = Gtk.Image.new_from_file(
-            os.path.join(get_pixmap_dir(), "pitivi-render-24.png"))
-        self.render_button = Gtk.Button()
-        self.render_button.set_image(render_icon)
+        self.render_button = Gtk.Button.new_from_icon_name(
+            "system-run-symbolic", Gtk.IconSize.SMALL_TOOLBAR)
         self.render_button.set_always_show_image(True)
         self.render_button.set_label(_("Render"))
         self.render_button.set_tooltip_text(
@@ -429,11 +402,27 @@ class MainWindow(Gtk.ApplicationWindow, Loggable):
         self.render_button.set_sensitive(False)  # The only one we have to set.
         self.render_button.connect("clicked", self._renderCb)
 
-        self._headerbar.pack_start(undo_button)
-        self._headerbar.pack_start(redo_button)
-        self._headerbar.pack_start(separator)
-        self._headerbar.pack_start(self.save_button)
-        self._headerbar.pack_start(self.render_button)
+        undo_redo_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=0)
+        undo_redo_box.get_style_context().add_class("linked")
+        undo_redo_box.pack_start(undo_button, expand=False, fill=False, padding=0)
+        undo_redo_box.pack_start(redo_button, expand=False, fill=False, padding=0)
+        self._headerbar.pack_start(undo_redo_box)
+
+        self.builder.add_from_file(
+            os.path.join(get_ui_dir(), "mainmenubutton.ui"))
+
+        # FIXME : see https://bugzilla.gnome.org/show_bug.cgi?id=729263
+        self.builder.connect_signals_full(self._builderConnectCb, self)
+
+        self._menubutton = self.builder.get_object("menubutton")
+
+        self._menubutton_items = {}
+        for widget in self.builder.get_object("menu").get_children():
+            self._menubutton_items[Gtk.Buildable.get_name(widget)] = widget
+
+        self._headerbar.pack_end(self._menubutton)
+        self._headerbar.pack_end(self.save_button)
+        self._headerbar.pack_end(self.render_button)
 
     def _set_keyboard_shortcuts(self):
         self.app.shortcuts.register_group("win", _("Project"), position=20)
@@ -604,20 +593,20 @@ class MainWindow(Gtk.ApplicationWindow, Loggable):
         abt.set_website(APPURL)
 
         if in_devel():
-            version_str = _("Development version: %s" % GITVERSION)
-        elif not self.app.isLatest():
-            version_str = _("Version %(cur_ver)s — %(new_ver)s is available" %
-                            {"cur_ver": GITVERSION,
-                             "new_ver": self.app.getLatest()})
-        elif GITVERSION:
-            version_str = _("Version %s" % GITVERSION)
+            version_str = _("Development version: %s") % GITVERSION
         else:
-            version_str = _("Version %s" % VERSION)
+            version = GITVERSION if GITVERSION.startswith(VERSION) else VERSION
+            if not self.app.isLatest():
+                version_str = _("Version %(cur_ver)s — %(new_ver)s is available") % \
+                    {"cur_ver": version,
+                     "new_ver": self.app.getLatest()}
+            else:
+                version_str = _("Version %s") % version
         abt.set_version(version_str)
 
         comments = ["",
                     "GES %s" % ".".join(map(str, GES.version())),
-                    "Gtk %s" % ".".join(map(str, (Gtk.MAJOR_VERSION, Gtk.MINOR_VERSION))),
+                    "GTK+ %s" % ".".join(map(str, (Gtk.MAJOR_VERSION, Gtk.MINOR_VERSION))),
                     "GStreamer %s" % ".".join(map(str, Gst.version()))]
         abt.set_comments("\n".join(comments))
 
@@ -646,6 +635,9 @@ class MainWindow(Gtk.ApplicationWindow, Loggable):
                      "Or you can run: git shortlog -s -n")
                    % "http://ohloh.net/p/pitivi/contributors", ]
         abt.set_authors(authors)
+        # Translators: See
+        # https://developer.gnome.org/gtk3/stable/GtkAboutDialog.html#gtk-about-dialog-set-translator-credits
+        # for details on how this is used.
         translators = _("translator-credits")
         if translators != "translator-credits":
             abt.set_translator_credits(translators)
@@ -812,12 +804,12 @@ class MainWindow(Gtk.ApplicationWindow, Loggable):
             last_saved = max(
                 os.path.getmtime(path), project_manager.time_loaded)
             time_delta = time() - last_saved
-            secondary.props.label = _("If you don't save, "
-                                      "the changes from the last %s will be lost."
-                                      % beautify_time_delta(time_delta))
+            message = _("If you don't save, "
+                        "the changes from the last %s will be lost.") % \
+                beautify_time_delta(time_delta)
         else:
-            secondary.props.label = _("If you don't save, "
-                                      "your changes will be lost.")
+            message = _("If you don't save, your changes will be lost.")
+        secondary.props.label = message
 
         # put the text in a vbox
         vbox = Gtk.Box(homogeneous=False, spacing=SPACING * 2)
@@ -918,7 +910,10 @@ class MainWindow(Gtk.ApplicationWindow, Loggable):
             # with more file-missing dialogs, as we need all of them.
             return None
 
-        uri = asset.get_id()
+        if self.app.proxy_manager.is_proxy_asset(asset):
+            uri = self.app.proxy_manager.getTargetUri(asset)
+        else:
+            uri = asset.get_id()
         dialog = Gtk.Dialog(title=_("Locate missing file..."),
                             transient_for=self,
                             modal=True)
@@ -930,45 +925,46 @@ class MainWindow(Gtk.ApplicationWindow, Loggable):
         dialog.set_transient_for(self)
         dialog.set_default_response(Gtk.ResponseType.OK)
 
-        # This box will contain the label and optionally a thumbnail
+        # This box will contain widgets with details about the missing file.
+        vbox = Gtk.Box()
+        vbox.set_orientation(Gtk.Orientation.VERTICAL)
+
+        label_start = Gtk.Label()
+        label_start.set_markup(_("The following file could not be found:"))
+        label_start.set_xalign(0)
+        vbox.pack_start(label_start, False, False, 0)
+
         hbox = Gtk.Box()
         hbox.set_orientation(Gtk.Orientation.HORIZONTAL)
-        hbox.set_spacing(SPACING)
+        hbox.set_margin_top(PADDING)
+        hbox.set_spacing(PADDING * 2)
 
-        # Check if we have a thumbnail available.
-        # This can happen if the file was moved or deleted by an application
-        # that does not manage Freedesktop thumbnails. The user is in luck!
-        # This is based on medialibrary's addDiscovererInfo method.
-        thumbnail_hash = md5(uri.encode()).hexdigest()
-        thumb_dir = os.path.expanduser("~/.thumbnails/normal/")
-        thumb_path_normal = thumb_dir + thumbnail_hash + ".png"
-        if os.path.exists(thumb_path_normal):
+        label_asset_info = Gtk.Label()
+        label_asset_info.set_markup(beautify_missing_asset(asset))
+        label_asset_info.set_xalign(0)
+        label_asset_info.set_yalign(0)
+        hbox.pack_start(label_asset_info, False, False, 0)
+
+        small_thumb, large_thumb = AssetThumbnail.get_thumbnails_from_xdg_cache(uri)
+        if large_thumb:
             self.debug("A thumbnail file was found for %s", uri)
-            thumbnail = Gtk.Image.new_from_file(thumb_path_normal)
-            thumbnail.set_padding(0, SPACING)
-            hbox.pack_start(thumbnail, False, False, 0)
+            thumbnail = Gtk.Image.new_from_pixbuf(large_thumb)
+            hbox.pack_end(thumbnail, False, False, 0)
 
-        # TODO: display the filesize to help the user identify the file
-        if asset.get_duration() == Gst.CLOCK_TIME_NONE:
-            # The file is probably an image, not video or audio.
-            text = _('The following file has moved: "<b>%s</b>"'
-                     '\nPlease specify its new location:'
-                     % info_name(asset))
-        else:
-            length = beautify_length(asset.get_duration())
-            text = _('The following file has moved: "<b>%s</b>" (duration: %s)'
-                     '\nPlease specify its new location:'
-                     % (info_name(asset), length))
+        vbox.pack_start(hbox, False, False, 0)
 
-        label = Gtk.Label()
-        label.set_markup(text)
-        hbox.pack_start(label, False, False, 0)
-        dialog.get_content_area().pack_start(hbox, False, False, 0)
-        hbox.show_all()
+        label_end = Gtk.Label()
+        label_end.set_markup(_("Please specify its new location:"))
+        label_end.set_xalign(0)
+        label_end.set_margin_top(PADDING)
+        vbox.pack_start(label_end, False, False, 0)
+
+        dialog.get_content_area().pack_start(vbox, False, False, 0)
+        vbox.show_all()
 
         chooser = Gtk.FileChooserWidget(action=Gtk.FileChooserAction.OPEN)
         chooser.set_select_multiple(False)
-        previewer = PreviewWidget(self.settings)
+        previewer = PreviewWidget(self.settings, discover_sync=True)
         chooser.set_preview_widget(previewer)
         chooser.set_use_preview_label(False)
         chooser.connect('update-preview', previewer.update_preview_cb)
@@ -980,7 +976,7 @@ class MainWindow(Gtk.ApplicationWindow, Loggable):
         filter_ = Gtk.FileFilter()
         # Translators: this is a format filter in a filechooser. Ex: "AVI
         # files"
-        filter_.set_name(_("%s files" % extension))
+        filter_.set_name(_("%s files") % extension)
         filter_.add_pattern("*%s" % extension.lower())
         filter_.add_pattern("*%s" % extension.upper())
         default = Gtk.FileFilter()
@@ -1010,8 +1006,8 @@ class MainWindow(Gtk.ApplicationWindow, Loggable):
                 # You have to do this *after* successfully creating a blank project,
                 # or the startupwizard will still be connected to that signal too.
                 reason = _('No replacement file was provided for "<i>%s</i>".\n\n'
-                           'Pitivi does not currently support partial projects.'
-                           % info_name(asset))
+                           'Pitivi does not currently support partial projects.') % \
+                    info_name(asset)
                 project_manager.emit("new-project-failed", project.uri, reason)
 
         dialog.destroy()
@@ -1021,13 +1017,11 @@ class MainWindow(Gtk.ApplicationWindow, Loggable):
         # FIXME GES we should re-enable this when possible
         # medialibrary.connect("missing-plugins", self._sourceListMissingPluginsCb)
         project.connect("project-changed", self._projectChangedCb)
-        project.connect(
-            "rendering-settings-changed", self._renderingSettingsChangedCb)
+        project.connect("rendering-settings-changed",
+                        self._rendering_settings_changed_cb)
         project.ges_timeline.connect("notify::duration",
                                      self._timelineDurationChangedCb)
 
-# Missing Plugins Support
-
     def _sourceListMissingPluginsCb(
         self, unused_project, unused_uri, unused_factory,
             details, unused_descriptions, missingPluginsCallback):
@@ -1043,8 +1037,6 @@ class MainWindow(Gtk.ApplicationWindow, Loggable):
                                                missingPluginsCallback)
         return res
 
-# Pitivi current project callbacks
-
     def _setProject(self, project):
         """Disconnects and then reconnects callbacks to the specified project.
 
@@ -1056,7 +1048,7 @@ class MainWindow(Gtk.ApplicationWindow, Loggable):
             return False
 
         self.viewer.setPipeline(project.pipeline)
-        self._renderingSettingsChangedCb(project)
+        self._reset_viewer_aspect_ratio(project)
         self.clipconfig.project = project
 
         # When creating a blank project there's no project URI yet.
@@ -1066,12 +1058,14 @@ class MainWindow(Gtk.ApplicationWindow, Loggable):
 
     def _disconnectFromProject(self, project):
         project.disconnect_by_func(self._projectChangedCb)
-        project.disconnect_by_func(self._renderingSettingsChangedCb)
+        project.disconnect_by_func(self._rendering_settings_changed_cb)
         project.ges_timeline.disconnect_by_func(self._timelineDurationChangedCb)
 
-# Pitivi current project callbacks
+    def _rendering_settings_changed_cb(self, project, unused_item):
+        """Handles Project metadata changes."""
+        self._reset_viewer_aspect_ratio(project)
 
-    def _renderingSettingsChangedCb(self, project, unused_item=None, unused_value=None):
+    def _reset_viewer_aspect_ratio(self, project):
         """Resets the viewer aspect ratio."""
         self.viewer.setDisplayAspectRatio(project.getDAR())
         self.viewer.timecode_entry.setFramerate(project.videorate)
@@ -1086,8 +1080,6 @@ class MainWindow(Gtk.ApplicationWindow, Loggable):
         self.debug("Timeline duration changed to %s", duration)
         self.render_button.set_sensitive(duration > 0)
 
-# other
-
     def _showExportDialog(self, project):
         self.log("Export requested")
         chooser = Gtk.FileChooserDialog(title=_("Export To..."),
@@ -1332,7 +1324,7 @@ class PreviewAssetWindow(Gtk.Window):
         # For videos and images, automatically resize the window
         # Try to keep it 1:1 if it can fit within 85% of the parent window
         video = video_streams[0]
-        img_width = video.get_width()
+        img_width = video.get_square_width()
         img_height = video.get_height()
         mainwindow_width, mainwindow_height = self._main_window.get_size()
         max_width = 0.85 * mainwindow_width
