@@ -46,10 +46,10 @@ from pitivi.settings import GlobalSettings
 from pitivi.timeline.previewers import ThumbnailCache
 from pitivi.utils.loggable import Loggable
 from pitivi.utils.misc import disconnectAllByFunc
-from pitivi.utils.misc import get_proxy_target
 from pitivi.utils.misc import path_from_uri
 from pitivi.utils.misc import PathWalker
 from pitivi.utils.misc import quote_uri
+from pitivi.utils.proxy import get_proxy_target
 from pitivi.utils.proxy import ProxyingStrategy
 from pitivi.utils.proxy import ProxyManager
 from pitivi.utils.ui import beautify_asset
@@ -59,6 +59,7 @@ from pitivi.utils.ui import FILE_TARGET_ENTRY
 from pitivi.utils.ui import fix_infobar
 from pitivi.utils.ui import info_name
 from pitivi.utils.ui import LARGE_THUMB_WIDTH
+from pitivi.utils.ui import PADDING
 from pitivi.utils.ui import SMALL_THUMB_WIDTH
 from pitivi.utils.ui import SPACING
 from pitivi.utils.ui import URI_TARGET_ENTRY
@@ -85,14 +86,13 @@ GlobalSettings.addConfigOption('lastClipView',
 
 STORE_MODEL_STRUCTURE = (
     GdkPixbuf.Pixbuf, GdkPixbuf.Pixbuf,
-    str, object, str, str, str, object)
+    str, object, str, str, object)
 
 (COL_ICON_64,
  COL_ICON_128,
  COL_INFOTEXT,
  COL_ASSET,
  COL_URI,
- COL_LENGTH,
  COL_SEARCH_TEXT,
  COL_THUMB_DECORATOR) = list(range(len(STORE_MODEL_STRUCTURE)))
 
@@ -210,14 +210,29 @@ class AssetThumbnail(Loggable):
         if video_streams:
             # Check if the files have thumbnails in the user's cache directory.
             real_uri = get_proxy_target(self.__asset).props.id
-            small_thumb, large_thumb = self.__get_thumbnails_from_xdg_cache(real_uri)
+            small_thumb, large_thumb = self.get_thumbnails_from_xdg_cache(real_uri)
             if not small_thumb:
                 if self.__asset.is_image():
-                    small_thumb, large_thumb = self.__get_icons("image-x-generic")
+                    path = Gst.uri_get_location(real_uri)
+                    try:
+                        pixbuf = GdkPixbuf.Pixbuf.new_from_file(path)
+                        width = pixbuf.props.width
+                        height = pixbuf.props.height
+                        small_thumb = pixbuf.scale_simple(
+                            SMALL_THUMB_WIDTH,
+                            SMALL_THUMB_WIDTH * height / width,
+                            GdkPixbuf.InterpType.BILINEAR)
+                        large_thumb = pixbuf.scale_simple(
+                            LARGE_THUMB_WIDTH,
+                            LARGE_THUMB_WIDTH * height / width,
+                            GdkPixbuf.InterpType.BILINEAR)
+                    except GLib.Error as error:
+                        self.debug("Failed loading thumbnail because: %s", error)
+                        small_thumb, large_thumb = self.__get_icons("image-x-generic")
                 else:
                     # Build or reuse a ThumbnailCache.
                     thumb_cache = ThumbnailCache.get(self.__asset)
-                    small_thumb = thumb_cache.getPreviewThumbnail()
+                    small_thumb = thumb_cache.get_preview_thumbnail()
                     if not small_thumb:
                         small_thumb, large_thumb = self.__get_icons("video-x-generic")
                     else:
@@ -236,7 +251,8 @@ class AssetThumbnail(Loggable):
             small_thumb, large_thumb = self.__get_icons("audio-x-generic")
         return small_thumb, large_thumb
 
-    def __get_thumbnails_from_xdg_cache(self, real_uri):
+    @staticmethod
+    def get_thumbnails_from_xdg_cache(real_uri):
         """Gets pixbufs for the specified thumbnail from the user's cache dir.
 
         Looks for thumbnails according to the [Thumbnail Managing Standard](https://specifications.freedesktop.org/thumbnail-spec/thumbnail-spec-latest.html#DIRECTORY).
@@ -250,8 +266,8 @@ class AssetThumbnail(Loggable):
         """
         quoted_uri = quote_uri(real_uri)
         thumbnail_hash = md5(quoted_uri.encode()).hexdigest()
-        thumb_dir = os.path.join(GLib.get_user_cache_dir(), "thumbnail")
-        path_128 = thumb_dir + "normal/" + thumbnail_hash + ".png"
+        thumb_dir = os.path.join(GLib.get_user_cache_dir(), "thumbnails")
+        path_128 = os.path.join(thumb_dir, "normal", thumbnail_hash + ".png")
         interpolation = GdkPixbuf.InterpType.BILINEAR
 
         # The cache dirs might have resolutions of 256 and/or 128,
@@ -264,7 +280,7 @@ class AssetThumbnail(Loggable):
             return small_thumb, large_thumb
         except GLib.GError:
             # path_128 doesn't exist, try the 256 version.
-            path_256 = thumb_dir + "large/" + thumbnail_hash + ".png"
+            path_256 = os.path.join(thumb_dir, "large", thumbnail_hash + ".png")
             try:
                 thumb_256 = GdkPixbuf.Pixbuf.new_from_file(path_256)
                 w, h = thumb_256.get_width(), thumb_256.get_height()
@@ -286,7 +302,7 @@ class AssetThumbnail(Loggable):
     def __get_icon(cls, icon_name, size):
         icon_theme = Gtk.IconTheme.get_default()
         try:
-            icon = icon_theme.load_icon(icon_name, size, 0)
+            icon = icon_theme.load_icon(icon_name, size, Gtk.IconLookupFlags.FORCE_SIZE)
         except GLib.Error:
             icon = icon_theme.load_icon("dialog-question", size, 0)
         return icon
@@ -294,7 +310,8 @@ class AssetThumbnail(Loggable):
     def __setState(self):
         asset = self.__asset
         target = asset.get_proxy_target()
-        if target and not target.get_error():
+        if self.proxy_manager.is_proxy_asset(asset) and target \
+                and not target.get_error():
             # The asset is a proxy.
             self.state = self.PROXIED
         elif asset.proxying_error:
@@ -349,7 +366,7 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
         Gtk.Box.__init__(self)
         Loggable.__init__(self)
 
-        self.pending_rows = []
+        self._pending_assets = []
 
         self.app = app
         self._errors = []
@@ -446,7 +463,9 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
         pixbufcol.set_spacing(SPACING)
         self.treeview.append_column(pixbufcol)
         pixcell = Gtk.CellRendererPixbuf()
-        pixcell.props.xpad = 6
+        pixcell.props.xpad = PADDING
+        pixcell.props.ypad = PADDING
+        pixcell.set_alignment(0, 0)
         pixbufcol.pack_start(pixcell, True)
         pixbufcol.add_attribute(pixcell, 'pixbuf', COL_ICON_64)
 
@@ -458,17 +477,10 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
         namecol.set_min_width(150)
         txtcell = Gtk.CellRendererText()
         txtcell.set_property("ellipsize", Pango.EllipsizeMode.START)
+        txtcell.set_alignment(0, 0)
         namecol.pack_start(txtcell, True)
         namecol.add_attribute(txtcell, "markup", COL_INFOTEXT)
 
-        namecol = Gtk.TreeViewColumn(_("Duration"))
-        namecol.set_expand(False)
-        self.treeview.append_column(namecol)
-        txtcell = Gtk.CellRendererText()
-        txtcell.set_property("yalign", 0.0)
-        namecol.pack_start(txtcell, True)
-        namecol.add_attribute(txtcell, "markup", COL_LENGTH)
-
         # IconView
         self.iconview = Gtk.IconView(model=self.modelFilter)
         self.iconview_scrollwin.add(self.iconview)
@@ -482,8 +494,8 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
         self.iconview.set_item_orientation(Gtk.Orientation.VERTICAL)
         self.iconview.set_property("has_tooltip", True)
         self.iconview.set_tooltip_column(COL_INFOTEXT)
-        self.iconview.props.item_padding = 3
-        self.iconview.props.margin = 3
+        self.iconview.props.item_padding = PADDING / 2
+        self.iconview.props.margin = PADDING / 2
         self.iconview_cursor_pos = None
 
         cell = Gtk.CellRendererPixbuf()
@@ -520,7 +532,7 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
                            [URI_TARGET_ENTRY, FILE_TARGET_ENTRY],
                            Gdk.DragAction.COPY)
         self.drag_dest_add_uri_targets()
-        self.connect("drag_data_received", self._dndDataReceivedCb)
+        self.connect("drag_data_received", self._drag_data_received_cb)
 
         self._setupViewAsDragAndDropSource(self.treeview)
         self._setupViewAsDragAndDropSource(self.iconview)
@@ -637,7 +649,8 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
         rows = [Gtk.TreeRowReference.new(model, path)
                 for path in paths]
 
-        with self.app.action_log.started("remove asset from media library"):
+        with self.app.action_log.started("remove asset from media library",
+                                         toplevel=True):
             for row in rows:
                 asset = model[row.get_path()][COL_ASSET]
                 target = asset.get_proxy_target()
@@ -774,42 +787,50 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
 
         self.debug("Adding asset %s", asset.props.id)
 
-        thumbs_decorator = AssetThumbnail(asset, self.app.proxy_manager)
-        duration = beautify_length(info.get_duration())
-        name = info_name(asset)
-        self.pending_rows.append((thumbs_decorator.small_thumb,
-                                  thumbs_decorator.large_thumb,
-                                  beautify_asset(asset),
-                                  asset,
-                                  asset.props.id,
-                                  duration,
-                                  name,
-                                  thumbs_decorator))
-        self._flushPendingRows()
+        self._pending_assets.append(asset)
 
-    def _flushPendingRows(self):
-        self.debug("Flushing %d pending model rows", len(self.pending_rows))
-        for row in self.pending_rows:
-            self.storemodel.append(row)
+        if self._project.loaded:
+            self._flushPendingAssets()
 
-        del self.pending_rows[:]
+    def _flushPendingAssets(self):
+        self.debug("Flushing %d pending model rows", len(self._pending_assets))
+        for asset in self._pending_assets:
+            thumbs_decorator = AssetThumbnail(asset, self.app.proxy_manager)
+            name = info_name(asset)
+
+            self.storemodel.append((thumbs_decorator.small_thumb,
+                                    thumbs_decorator.large_thumb,
+                                    beautify_asset(asset),
+                                    asset,
+                                    asset.props.id,
+                                    name,
+                                    thumbs_decorator))
+
+        del self._pending_assets[:]
 
     # medialibrary callbacks
 
     def _assetLoadingProgressCb(self, project, progress, estimated_time):
         self._progressbar.set_fraction(progress / 100)
 
+        proxying_files = []
         for row in self.storemodel:
-            row[COL_INFOTEXT] = beautify_asset(row[COL_ASSET])
+            asset = row[COL_ASSET]
+            row[COL_INFOTEXT] = beautify_asset(asset)
+
+            if not asset.ready:
+                proxying_files.append(asset)
+                if row[COL_THUMB_DECORATOR].state != AssetThumbnail.IN_PROGRESS:
+                    thumbs_decorator = AssetThumbnail(asset, self.app.proxy_manager)
+                    row[COL_ICON_64] = thumbs_decorator.small_thumb
+                    row[COL_ICON_128] = thumbs_decorator.large_thumb
+                    row[COL_THUMB_DECORATOR] = thumbs_decorator
 
         if progress == 0:
             self._startImporting(project)
             return
 
         if project.loaded:
-            num_proxying_files = [asset
-                                  for asset in project.loading_assets
-                                  if not asset.ready]
             if estimated_time:
                 self.__last_proxying_estimate_time = beautify_ETA(int(
                     estimated_time * Gst.SECOND))
@@ -822,9 +843,9 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
             # "There remains approximatively %s" (to handle gender and plurals)
             template = ngettext("Transcoding %d asset: %d%% (About %s left)",
                                 "Transcoding %d assets: %d%% (About %s left)",
-                                len(num_proxying_files))
+                                len(proxying_files))
             progress_message = template % (
-                len(num_proxying_files), progress,
+                len(proxying_files), progress,
                 self.__last_proxying_estimate_time)
             self._progressbar.set_text(progress_message)
             self._last_imported_uris.update([asset.props.id for asset in
@@ -834,7 +855,14 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
             self._doneImporting()
 
     def __assetProxyingCb(self, proxy, unused_pspec):
-        self.debug("Proxy is %s", proxy.props.id)
+        if not self.app.proxy_manager.is_proxy_asset(proxy):
+            self.info("Proxy is not a proxy in our terms (handling deleted proxy"
+                      " files while loading a project?) - ignore it")
+
+            return
+
+        self.debug("Proxy is %s - %s", proxy.props.id,
+                   proxy.get_proxy_target())
         self.__removeAsset(proxy)
 
         if proxy.get_proxy_target() is not None:
@@ -915,7 +943,7 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
     def _doneImporting(self):
         self.debug("Importing took %.3f seconds",
                    time.time() - self.import_start_time)
-        self._flushPendingRows()
+        self._flushPendingAssets()
         self._progressbar.hide()
         if self._errors:
             errors_amount = len(self._errors)
@@ -961,6 +989,9 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
         self.debug("response: %r", response)
         if response == Gtk.ResponseType.OK:
             lastfolder = dialogbox.get_current_folder()
+            # get_current_folder() is None if file was chosen from 'Recents'
+            if not lastfolder:
+                lastfolder = GLib.path_get_dirname(dialogbox.get_filename())
             self.app.settings.lastImportFolder = lastfolder
             dialogbox.props.extra_widget.saveValues()
             filenames = dialogbox.get_uris()
@@ -1035,7 +1066,7 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
 
     def __warningInfobarCb(self, infobar, response_id):
         if response_id == Gtk.ResponseType.OK:
-            self.__showErrors()
+            self.__show_errors()
         self._resetErrorList()
         infobar.hide()
 
@@ -1043,15 +1074,15 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
         self._errors = []
         self._import_warning_infobar.hide()
 
-    def __showErrors(self):
+    def __show_errors(self):
         """Shows a dialog with the import errors."""
-        if len(self._errors) > 1:
-            msgs = (_("Error while analyzing files"),
-                    _("The following files can not be used with Pitivi."))
-        else:
-            msgs = (_("Error while analyzing a file"),
-                    _("The following file can not be used with Pitivi."))
-        error_dialogbox = FileListErrorDialog(*msgs)
+        title = ngettext("Error while analyzing a file",
+                         "Error while analyzing files",
+                         len(self._errors))
+        headline = ngettext("The following file can not be used with Pitivi.",
+                            "The following files can not be used with Pitivi.",
+                            len(self._errors))
+        error_dialogbox = FileListErrorDialog(title, headline)
         error_dialogbox.connect("close", self._errorDialogBoxCloseCb)
         error_dialogbox.connect("response", self._errorDialogBoxResponseCb)
 
@@ -1066,7 +1097,12 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
         else:
             self._setClipView(SHOW_ICONVIEW)
 
-    def __getPathUnderMouse(self, view, event):
+    def __get_path_under_mouse(self, view, event):
+        """Gets the path of the item under the mouse cursor.
+
+        Returns:
+            Gtk.TreePath: The item at the current mouse position, if any.
+        """
         if isinstance(view, Gtk.TreeView):
             path = None
             tup = view.get_path_at_pos(int(event.x), int(event.y))
@@ -1076,30 +1112,19 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
         elif isinstance(view, Gtk.IconView):
             return view.get_path_at_pos(int(event.x), int(event.y))
         else:
-            raise RuntimeError(
-                "Unknown media library view type: %s" % type(view))
+            raise RuntimeError("Unknown view type: %s" % type(view))
 
     def _rowUnderMouseSelected(self, view, event):
-        path = self.__getPathUnderMouse(view, event)
+        path = self.__get_path_under_mouse(view, event)
+        if not path:
+            return False
         if isinstance(view, Gtk.TreeView):
-            path = None
-            tup = view.get_path_at_pos(int(event.x), int(event.y))
-            if tup:
-                path, column, x, y = tup
-            if path:
-                selection = view.get_selection()
-
-                return selection.path_is_selected(path) and selection.count_selected_rows() > 0
+            tree_selection = view.get_selection()
+            return tree_selection.path_is_selected(path)
         elif isinstance(view, Gtk.IconView):
-            path = view.get_path_at_pos(int(event.x), int(event.y))
-            if path:
-                selection = view.get_selected_items()
-                return view.path_is_selected(path) and len(selection)
+            return view.path_is_selected(path)
         else:
-            raise RuntimeError(
-                "Unknown media library view type: %s" % type(view))
-
-        return False
+            raise RuntimeError("Unknown view type: %s" % type(view))
 
     def _viewGetFirstSelected(self):
         paths = self.getSelectedPaths()
@@ -1131,16 +1156,22 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
         elif self.clip_view == SHOW_ICONVIEW:
             self.iconview.unselect_all()
 
-    def __stopUsingProxyCb(self,
-                           unused_action,
-                           unused_parameter):
-        self._project.disableProxiesForAssets(self.getSelectedAssets())
+    def __stopUsingProxyCb(self, unused_action, unused_parameter):
+        self._project.disable_proxies_for_assets(self.getSelectedAssets())
 
     def __useProxiesCb(self, unused_action, unused_parameter):
-        self._project.useProxiesForAssets(self.getSelectedAssets())
+        self._project.use_proxies_for_assets(self.getSelectedAssets())
 
     def __deleteProxiesCb(self, unused_action, unused_parameter):
-        self._project.disableProxiesForAssets(self.getSelectedAssets(), delete_proxy_file=True)
+        self._project.disable_proxies_for_assets(self.getSelectedAssets(),
+                                                 delete_proxy_file=True)
+
+    def __open_containing_folder_cb(self, unused_action, unused_parameter):
+        assets = self.getSelectedAssets()
+        if len(assets) != 1:
+            return
+        parent_path = os.path.dirname(path_from_uri(assets[0].get_id()))
+        Gio.AppInfo.launch_default_for_uri(Gst.filename_to_uri(parent_path), None)
 
     def __createMenuModel(self):
         if self.app.proxy_manager.proxyingUnsupported:
@@ -1153,8 +1184,15 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
         action_group = Gio.SimpleActionGroup()
         menu_model = Gio.Menu()
 
+        if len(assets) == 1:
+            action = Gio.SimpleAction.new("open-folder", None)
+            action.connect("activate", self.__open_containing_folder_cb)
+            action_group.insert(action)
+            text = _("Open containing folder")
+            menu_model.append(text, "assets.%s" % action.get_name().replace(" ", "."))
+
         proxies = [asset.get_proxy_target() for asset in assets
-                   if asset.get_proxy_target()]
+                   if self.app.proxy_manager.is_proxy_asset(asset)]
         in_progress = [asset.creation_progress for asset in assets
                        if asset.creation_progress < 100]
 
@@ -1198,7 +1236,7 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
             return False
 
         if not self._rowUnderMouseSelected(view, event):
-            path = self.__getPathUnderMouse(view, event)
+            path = self.__get_path_under_mouse(view, event)
             if path:
                 if isinstance(view, Gtk.IconView):
                     view.unselect_all()
@@ -1350,7 +1388,7 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
 
     def _newProjectLoadedCb(self, unused_project_manager, project):
         # Make sure that the sources added to the project are added
-        self._flushPendingRows()
+        self._flushPendingAssets()
 
     def _newProjectFailedCb(self, unused_project_manager, unused_uri, unused_reason):
         self.storemodel.clear()
@@ -1362,54 +1400,29 @@ class MediaLibraryWidget(Gtk.Box, Loggable):
         self.storemodel.clear()
         self._project = None
 
-    def _addUris(self, uris):
-        if self._project:
-            self._project.addUris(uris)
+    def __paths_walked_cb(self, uris):
+        """Handles the end of the path walking when importing files and dirs."""
+        if not uris:
+            return
+        if not self._project:
+            self.warning("Cannot add URIs, project missing")
+        self._last_imported_uris = set(uris)
+        assets = self._project.assetsForUris(uris)
+        if assets:
+            # All the files have already been added.
+            self._selectLastImportedUris()
         else:
-            self.warning(
-                "Adding uris to project, but the project has changed in the meantime")
-        return False
+            self._project.addUris(uris)
 
-    # Drag and Drop
-    def _dndDataReceivedCb(self, unused_widget, unused_context, unused_x,
-                           unused_y, selection, targettype, unused_time):
+    def _drag_data_received_cb(self, unused_widget, unused_context, unused_x,
+                               unused_y, selection, targettype, unused_time):
+        """Handles data being dragged onto self."""
         self.debug("targettype: %d, selection.data: %r",
                    targettype, selection.get_data())
-
-        directories = []
-        filenames = []
-
         uris = selection.get_uris()
-        # Filter out the empty uris.
-        uris = [x for x in uris if x]
-        for raw_uri in uris:
-            # Strip out NULL chars first.
-            raw_uri = raw_uri.strip('\x00')
-            uri = urlparse(raw_uri)
-            if uri.scheme == 'file':
-                path = unquote(uri.path)
-                if os.path.isfile(path):
-                    filenames.append(raw_uri)
-                elif os.path.isdir(path):
-                    directories.append(raw_uri)
-                else:
-                    self.warning("Unusable file: %s, %s", raw_uri, path)
-            else:
-                self.fixme(
-                    "Importing remote files is not implemented: %s", raw_uri)
-
-        if directories:
-            # Recursively import from folders that were dragged into the
-            # library
-            self.app.threads.addThread(PathWalker, directories, self._addUris)
-        if filenames:
-            self._last_imported_uris.update(filenames)
-            assets = self._project.assetsForUris(list(self._last_imported_uris))
-            if assets:
-                # All the files have already been added.
-                self._selectLastImportedUris()
-            else:
-                self._project.addUris(filenames)
+        # Scan in the background what was dragged and
+        # import whatever can be imported.
+        self.app.threads.addThread(PathWalker, uris, self.__paths_walked_cb)
 
     # Used with TreeView and IconView
     def _dndDragDataGetCb(self, unused_view, unused_context, data, unused_info, unused_timestamp):
