@@ -77,7 +77,9 @@ class TrackElementPropertyChanged(UndoableAction):
         if isinstance(self.new_value, GObject.GFlags) or\
                 isinstance(self.new_value, GObject.GEnum):
             value = int(self.new_value)
-        st['value'] = value
+
+        _, _, pspec = self.track_element.lookup_child(self.property_name)
+        st['value'] = GObject.Value(pspec.value_type, value)
         return st
 
 
@@ -144,7 +146,11 @@ class TrackElementObserver(TimelineElementObserver):
 
     def __init__(self, ges_track_element, action_log):
         TimelineElementObserver.__init__(self, ges_track_element, action_log)
-        self.gobject_observer = GObjectObserver(ges_track_element, ("active",), action_log)
+        if isinstance(ges_track_element, GES.BaseEffect):
+            property_names = ("active", "priority",)
+        else:
+            property_names = ("active",)
+        self.gobject_observer = GObjectObserver(ges_track_element, property_names, action_log)
 
     def release(self):
         TimelineElementObserver.release(self)
@@ -159,7 +165,7 @@ class TrackElementAction(UndoableAction):
         self.track_element = track_element
         self.track_element_props = []
         for prop in self.track_element.list_children_properties():
-            if not prop.flags & GObject.PARAM_WRITABLE or \
+            if not prop.flags & GObject.ParamFlags.WRITABLE or \
                     prop.name in PROPS_TO_IGNORE:
                 continue
             prop_name = child_property_name(prop)
@@ -633,6 +639,7 @@ class ControlSourceRemoveAction(UndoableAction):
         st.set_value("property-name", self.property_name)
         return st
 
+
 class LayerObserver(MetaContainerObserver, Loggable):
     """Monitors a Layer and reports UndoableActions.
 
