@@ -1,7 +1,5 @@
-#!/usr/bin/env python3
-#
-#       pitivi/utils/pipeline.py
-#
+# -*- coding: utf-8 -*-
+# Pitivi video editor
 # Copyright (C) 2012 Thibault Saunier <thibault.saunier@collabora.com>
 #
 # This program is free software; you can redistribute it and/or
@@ -20,20 +18,17 @@
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 3, or (at your option)
 # any later version.
-
-"""
-High-level pipelines
-"""
+"""High-level pipelines."""
 import os
 
+from gi.repository import GES
 from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gst
-from gi.repository import GES
 
+from pitivi.check import videosink_factory
 from pitivi.utils.loggable import Loggable
 from pitivi.utils.misc import format_ns
-from pitivi.check import videosink_factory
 
 
 PIPELINE_SIGNALS = {
@@ -43,6 +38,7 @@ PIPELINE_SIGNALS = {
     "eos": (GObject.SignalFlags.RUN_LAST, None, ()),
     "error": (GObject.SignalFlags.RUN_LAST, None, (GObject.TYPE_STRING, GObject.TYPE_STRING)),
     "died": (GObject.SignalFlags.RUN_LAST, None, ()),
+    "async-done": (GObject.SignalFlags.RUN_LAST, None, ()),
 }
 
 MAX_RECOVERIES = 3
@@ -50,120 +46,30 @@ WATCHDOG_TIMEOUT = 3
 MAX_BRINGING_TO_PAUSED_DURATION = 5
 MAX_SET_STATE_DURATION = 1
 
+DEFAULT_POSITION_LISTENNING_INTERVAL = 500
+
 
 class PipelineError(Exception):
     pass
 
 
-class Seeker(GObject.Object, Loggable):
-
-    """
-    The Seeker is a singleton helper class to do various seeking
-    operations in the pipeline.
-    """
-
-    _instance = None
-
-    __gsignals__ = {
-        "seek": (GObject.SIGNAL_RUN_LAST, None, (GObject.TYPE_UINT64,)),
-        "seek-relative": (GObject.SIGNAL_RUN_LAST, None, (GObject.TYPE_INT64,)),
-    }
-
-    def __new__(cls, *args, **kwargs):
-        """
-        Override the new method to return the singleton instance if available.
-        Otherwise, create one.
-        """
-        if not cls._instance:
-            cls._instance = super(Seeker, cls).__new__(cls, *args, **kwargs)
-        return cls._instance
-
-    def __init__(self, timeout=80):
-        """
-        @param timeout (optional): the amount of miliseconds for a seek attempt
-        """
-        GObject.Object.__init__(self)
-        Loggable.__init__(self)
-
-        self.timeout = timeout
-        self.pending_seek_id = None
-        self.position = None
-        self._time = None
-        self.pending_position = None
-
-    def seek(self, position, on_idle=False):
-        self.position = position
-
-        if self.pending_seek_id is None:
-            if on_idle:
-                self.pending_seek_id = self._scheduleSeek(
-                    self.timeout, self._seekTimeoutCb)
-            else:
-                self._seekTimeoutCb()
-        else:
-            self.pending_position = position
-
-    def seekRelative(self, time, on_idle=False):
-        if self.pending_seek_id is None:
-            self._time = int(time)
-            if on_idle:
-                self.pending_seek_id = self._scheduleSeek(
-                    self.timeout, self._seekTimeoutCb, relative=True)
-            else:
-                self._seekTimeoutCb(relative=True)
-
-    def flush(self, on_idle=False):
-        self.seekRelative(0, on_idle)
-
-    def _scheduleSeek(self, timeout, callback, relative=False):
-        return GLib.timeout_add(timeout, callback, relative)
-
-    def _seekTimeoutCb(self, relative=False):
-        self.pending_seek_id = None
-
-        if relative:
-            try:
-                self.emit('seek-relative', self._time)
-            except PipelineError:
-                self.error("Error while seeking %s relative", self._time)
-                # if an exception happened while seeking, properly
-                # reset ourselves
-                return False
-
-            self._time = None
-        elif self.position is not None:
-            position = max(0, self.position)
-            self.position = None
-            try:
-                self.emit('seek', position)
-            except PipelineError as e:
-                self.error("Error while seeking to position: %s, reason: %s",
-                           format_ns(position), e)
-                # if an exception happened while seeking, properly
-                # reset ourselves
-                return False
-
-        if self.pending_position:
-            self.seek(self.pending_position, on_idle=True)
-            self.pending_position = None
-
-        return False
-
-
 class SimplePipeline(GObject.Object, Loggable):
+    """High-level pipeline.
 
-    """
-    The Pipeline is only responsible for:
+    The `SimplePipeline` is responsible for:
      - State changes
      - Position seeking
-     - Position Querying
-       - Along with an periodic callback (optional)
+     - Position querying
+     - Along with an periodic callback (optional)
 
     Signals:
-     - C{state-change} : The state of the pipeline changed.
-     - C{position} : The current position of the pipeline changed.
-     - C{eos} : The Pipeline has finished playing.
-     - C{error} : An error happened.
+        state-change: The state of the pipeline changed.
+        position: The current position of the pipeline changed.
+        eos: The Pipeline has finished playing.
+        error: An error happened.
+
+    Attributes:
+        _pipeline (Gst.Pipeline): The low-level pipeline.
     """
 
     __gsignals__ = PIPELINE_SIGNALS
@@ -182,49 +88,46 @@ class SimplePipeline(GObject.Object, Loggable):
         self._bus.add_signal_watch()
         self._bus.connect("message", self._busMessageCb)
         self._listening = False  # for the position handler
-        self._listeningInterval = 50  # default 300ms
+        self._listeningInterval = DEFAULT_POSITION_LISTENNING_INTERVAL
         self._listeningSigId = 0
         self._duration = Gst.CLOCK_TIME_NONE
-        self._last_position = int(0 * Gst.SECOND)
+        # The last known position.
+        self._last_position = 0 * Gst.SECOND
         self._recovery_state = self.RecoveryState.NOT_RECOVERING
         self._attempted_recoveries = 0
-        self._waiting_for_async_done = False
+        # The position where the user intends to seek.
         self._next_seek = None
         self._timeout_async_id = 0
         self._force_position_listener = False
 
-        self.setSink(self.createSink())
+        self.video_sink = None
+        self.sink_widget = None
 
-    def createSink(self):
+    def create_sink(self):
         sink = Gst.ElementFactory.make(videosink_factory.get_name(), None)
+        self.sink_widget = sink.props.widget
+
         if videosink_factory.get_name() == "gtksink":
             self.info("Using gtksink")
-            return sink
-
-        sinkbin = Gst.ElementFactory.make("glsinkbin", None)
-        sinkbin.props.sink = sink
-        return sinkbin
-
-    def setSink(self, sink):
-        self.video_sink = sink
-        if isinstance(self._pipeline, GES.Pipeline):
-            self._pipeline.preview_set_video_sink(self.video_sink)
+            self.video_sink = sink
         else:
-            self._pipeline.set_property("video_sink", self.video_sink)
+            self.info("Using glsinkbin around %s", videosink_factory.get_name())
+            sinkbin = Gst.ElementFactory.make("glsinkbin", None)
+            sinkbin.props.sink = sink
+            self.video_sink = sinkbin
 
     def setForcePositionListener(self, force):
         self._force_position_listener = force
 
     def release(self):
-        """
-        Release the L{Pipeline} and all used L{ObjectFactory} and
-        L{Action}s.
+        """Releases the low-level pipeline.
 
-        Call this method when the L{Pipeline} is no longer used. Forgetting to do
-        so will result in memory loss.
+        Call this method when this instance is no longer used. Forgetting to do
+        so will result in memory leaks.
 
-        @postcondition: The L{Pipeline} will no longer be usable.
+        The instance will no longer be usable.
         """
+        self._removeWaitingForAsyncDoneTimeout()
         self.deactivatePositionListener()
         self._bus.disconnect_by_func(self._busMessageCb)
         self._bus.remove_signal_watch()
@@ -238,19 +141,20 @@ class SimplePipeline(GObject.Object, Loggable):
             return
 
         try:
-            self.seekRelative(0)
+            self.simple_seek(self.getPosition())
         except PipelineError as e:
             self.warning("Could not flush because: %s", e)
             pass
 
     def setState(self, state):
-        """
-        Set the L{Pipeline} to the given state.
+        """Sets the low-level pipeline to the specified state.
 
-        @raises PipelineError: If the C{Gst.Pipeline} could not be changed to
-        the requested state.
+        Raises:
+            PipelineError: If the low-level pipeline could not be changed to
+                the requested state. In this case the state is set to
+                Gst.State.NULL
         """
-        self.debug("state set to: %r", state)
+        self.debug("Setting state to: %r", state)
         if state >= Gst.State.PAUSED:
             cstate = self.getState()
             if cstate < Gst.State.PAUSED:
@@ -271,14 +175,12 @@ class SimplePipeline(GObject.Object, Loggable):
                 "Failure changing state of the Gst.Pipeline to %r, currently reset to NULL" % state)
 
     def getState(self):
-        """
-        Query the L{Pipeline} for the current state.
-
-        @see: L{setState}
+        """Queries the low-level pipeline for the current state.
 
         This will do an actual query to the underlying GStreamer Pipeline.
-        @return: The current state.
-        @rtype: C{State}
+
+        Returns:
+            State: The current state.
         """
         # No timeout
         change, state, pending = self._pipeline.get_state(timeout=0)
@@ -287,61 +189,56 @@ class SimplePipeline(GObject.Object, Loggable):
         return state
 
     def play(self):
-        """
-        Sets the L{Pipeline} to PLAYING
-        """
+        """Sets the state to Gst.State.PLAYING."""
         self.setState(Gst.State.PLAYING)
 
     def pause(self):
-        """
-        Sets the L{Pipeline} to PAUSED
-        """
+        """Sets the state to Gst.State.PAUSED."""
         self.setState(Gst.State.PAUSED)
 
     def stop(self):
-        """
-        Sets the L{Pipeline} to READY
-        """
+        """Sets the state to Gst.State.READY."""
         self.setState(Gst.State.READY)
 
+    def playing(self):
+        return self.getState() == Gst.State.PLAYING
+
     def togglePlayback(self):
-        if self.getState() == Gst.State.PLAYING:
+        if self.playing():
             self.pause()
         else:
             self.play()
 
     # Position and Seeking methods
 
-    def getPosition(self, blocks=False):
-        """
-        Get the current position of the L{Pipeline}.
+    def getPosition(self, fails=True):
+        """Gets the current position of the low-level pipeline.
 
-        @return: The current position or Gst.CLOCK_TIME_NONE
-        @rtype: L{long}
-        @raise PipelineError: If the position couldn't be obtained.
-        """
-        maincontext = GLib.main_context_default()
-        if blocks and self._recovery_state == self.RecoveryState.NOT_RECOVERING:
-            while self._waiting_for_async_done and self._recovery_state == self.RecoveryState.NOT_RECOVERING:
-                self.info("Iterating mainloop waiting for the pipeline to be ready to be queried")
-                maincontext.iteration(True)
+        Returns:
+            int: The current position or Gst.CLOCK_TIME_NONE.
 
+        Raises:
+            PipelineError: If the position couldn't be obtained.
+        """
         try:
             res, cur = self._pipeline.query_position(Gst.Format.TIME)
         except Exception as e:
             self.handleException(e)
             raise PipelineError("Couldn't get position")
 
-        if not res:
-            raise PipelineError("Position not available")
+        if res:
+            self._last_position = cur
+        else:
+            if fails:
+                raise PipelineError("Position not available")
+
+            cur = self._last_position
 
         self.log("Got position %s", format_ns(cur))
         return cur
 
     def getDuration(self):
-        """
-        Get the duration of the C{Pipeline}.
-        """
+        """Gets the duration of the low-level pipeline."""
         dur = self._getDuration()
         self.log("Got duration %s", format_ns(dur))
         if self._duration != dur:
@@ -349,18 +246,17 @@ class SimplePipeline(GObject.Object, Loggable):
         self._duration = dur
         return dur
 
-    def activatePositionListener(self, interval=50):
-        """
-        Activate the position listener.
+    def activatePositionListener(self, interval=DEFAULT_POSITION_LISTENNING_INTERVAL):
+        """Activates the position listener.
 
-        When activated, the Pipeline will emit the 'position' signal at the
+        When activated, the instance will emit the `position` signal at the
         specified interval when it is the PLAYING or PAUSED state.
 
-        @see: L{deactivatePositionListener}
-        @param interval: Interval between position queries in milliseconds
-        @type interval: L{int} milliseconds
-        @return: Whether the position listener was activated or not
-        @rtype: L{bool}
+        Args:
+            interval (int): Interval between position queries in milliseconds.
+
+        Returns:
+            bool: Whether the position listener was activated.
         """
         if self._listening:
             return True
@@ -371,11 +267,7 @@ class SimplePipeline(GObject.Object, Loggable):
         return True
 
     def deactivatePositionListener(self):
-        """
-        De-activates the position listener.
-
-        @see: L{activatePositionListener}
-        """
+        """De-activates the position listener."""
         self._listenToPosition(False)
         self._listening = False
 
@@ -387,8 +279,7 @@ class SimplePipeline(GObject.Object, Loggable):
                 self.warning("Could not get position because: %s", e)
             else:
                 if position != Gst.CLOCK_TIME_NONE:
-                    self.emit('position', position)
-                    self._last_position = position
+                    self.emit("position", position)
         finally:
             return True
 
@@ -404,71 +295,93 @@ class SimplePipeline(GObject.Object, Loggable):
             GLib.source_remove(self._listeningSigId)
             self._listeningSigId = 0
 
-    def _asyncDoneNotReceivedCb(self):
+    def _async_done_not_received_cb(self):
         self.error("we didn't get async done, this is a bug")
-        self._recover()
-        # Source is being removed
         self._removeWaitingForAsyncDoneTimeout()
+        self._recover()
         return False
 
     def _removeWaitingForAsyncDoneTimeout(self):
-        if self._timeout_async_id:
-            GLib.source_remove(self._timeout_async_id)
+        if not self._busy_async:
+            return
+        GLib.source_remove(self._timeout_async_id)
         self._timeout_async_id = 0
 
     def _addWaitingForAsyncDoneTimeout(self, timeout=WATCHDOG_TIMEOUT):
         self._removeWaitingForAsyncDoneTimeout()
-
         self._timeout_async_id = GLib.timeout_add_seconds(timeout,
-                                                          self._asyncDoneNotReceivedCb)
-        self._waiting_for_async_done = True
+                                                          self._async_done_not_received_cb)
+
+    @property
+    def _busy_async(self):
+        """Gets whether the pipeline is busy in the background.
+
+        The following operations are performed in the background:
+        - State changing from READY to PAUSED. For example a pipeline in
+          NULL set to PLAYING, goes through each intermediary state
+          including READY to PAUSED, so we consider it ASYNC.
+        - Seeking.
+        - Committing, but only if the timeline is not empty at the time of the commit.
+
+        When the pipeline is working in the background, no seek nor commit
+        should be performed.
+
+        Returns:
+            bool: True iff the pipeline is busy.
+        """
+        return bool(self._timeout_async_id)
 
     def simple_seek(self, position):
-        """
-        Seeks in the L{Pipeline} to the given position.
+        """Seeks in the low-level pipeline to the specified position.
 
-        @param position: Position to seek to
-        @type position: L{long}
-        @raise PipelineError: If seek failed
+        Args:
+            position (int): Position to seek to.
+
+        Raises:
+            PipelineError: When the seek fails.
         """
-        if self._waiting_for_async_done is True:
+        if self._busy_async or self.getState() < Gst.State.PAUSED:
             self._next_seek = position
             self.info("Setting next seek to %s", self._next_seek)
             return
 
-        self.debug("position: %s", format_ns(position))
+        self._next_seek = None
 
         # clamp between [0, duration]
         position = max(0, min(position, self.getDuration()))
-
-        res = self._pipeline.seek(1.0, Gst.Format.TIME, Gst.SeekFlags.FLUSH | Gst.SeekFlags.ACCURATE,
-                                  Gst.SeekType.SET, position,
-                                  Gst.SeekType.NONE, -1)
-        self._addWaitingForAsyncDoneTimeout()
-
+        self.debug("Seeking to position: %s", format_ns(position))
+        res = self._pipeline.seek(1.0,
+                                  Gst.Format.TIME,
+                                  Gst.SeekFlags.FLUSH | Gst.SeekFlags.ACCURATE,
+                                  Gst.SeekType.SET,
+                                  position,
+                                  Gst.SeekType.NONE,
+                                  -1)
         if not res:
             raise PipelineError(self.get_name() + " seek failed: " + str(position))
 
-        self._last_position = position
+        self._addWaitingForAsyncDoneTimeout()
 
-        self.debug("seeking successful")
         self.emit('position', position)
 
     def seekRelative(self, time_delta):
-        self.simple_seek(self.getPosition() + time_delta)
+        try:
+            self.simple_seek(self.getPosition() + int(time_delta))
+        except PipelineError:
+            self.error("Error while seeking %s relative", time_delta)
 
     # Private methods
 
     def _busMessageCb(self, unused_bus, message):
         if message.type == Gst.MessageType.EOS:
+            self.__emitPosition()
             self.pause()
             self.emit('eos')
         elif message.type == Gst.MessageType.STATE_CHANGED:
-            prev, new, pending = message.parse_state_changed()
-
             if message.src == self._pipeline:
+                prev, new, pending = message.parse_state_changed()
                 self.debug(
-                    "Pipeline change state prev: %r, new: %r, pending: %r", prev, new, pending)
+                    "Pipeline changed state. prev: %r, new: %r, pending: %r", prev, new, pending)
 
                 emit_state_change = pending == Gst.State.VOID_PENDING
                 if prev == Gst.State.READY and new == Gst.State.PAUSED:
@@ -484,13 +397,15 @@ class SimplePipeline(GObject.Object, Loggable):
                         if self._attempted_recoveries == MAX_RECOVERIES:
                             self._recovery_state = self.RecoveryState.NOT_RECOVERING
                             self._attempted_recoveries = 0
-                            self.error("Too many tries to seek back to right position"
+                            self.error("Too many tries to seek back to right position, "
                                        "not trying again, and going back to 0 instead")
                         else:
+                            self.info("Performing seek after pipeline recovery")
                             self._recovery_state = self.RecoveryState.SEEKED_AFTER_RECOVERING
-                            self.simple_seek(self._last_position)
-                            self.info(
-                                "Seeked back to the last position after pipeline recovery")
+                            position = self._last_position
+                            if self._next_seek is not None:
+                                position = self._next_seek
+                            self.simple_seek(position)
                     self._listenToPosition(self._force_position_listener)
                 elif prev == Gst.State.PAUSED and new == Gst.State.PLAYING:
                     self._listenToPosition(True)
@@ -507,28 +422,30 @@ class SimplePipeline(GObject.Object, Loggable):
                                               Gst.DebugGraphDetails.ALL,
                                               "pitivi.error")
             if not self._rendering():
+                self._removeWaitingForAsyncDoneTimeout()
                 self._recover()
         elif message.type == Gst.MessageType.DURATION_CHANGED:
-            self.debug("Duration might have changed, querying it")
+            self.debug("Querying duration async, because it changed")
             GLib.idle_add(self._queryDurationAsync)
         elif message.type == Gst.MessageType.ASYNC_DONE:
+            self.debug("Async done, ready for action")
+            self.emit("async-done")
+            self._removeWaitingForAsyncDoneTimeout()
             if self._recovery_state == self.RecoveryState.SEEKED_AFTER_RECOVERING:
                 self._recovery_state = self.RecoveryState.NOT_RECOVERING
                 self._attempted_recoveries = 0
-            self._waiting_for_async_done = False
             self.__emitPosition()
             if self._next_seek is not None:
+                self.info("Performing seek after ASYNC_DONE")
                 self.simple_seek(self._next_seek)
-                self._next_seek = None
-            self._removeWaitingForAsyncDoneTimeout()
         else:
             self.log("%s [%r]", message.type, message.src)
 
     def __emitPosition(self):
         # When the pipeline has been paused we need to update the
         # timeline/playhead position, as the 'position' signal
-        # is only emitted every 300ms and the playhead jumps
-        # during the playback.
+        # is only emitted every DEFAULT_POSITION_LISTENNING_INTERVAL
+        # ms and the playhead jumps during the playback.
         try:
             position = self.getPosition()
         except PipelineError as e:
@@ -540,30 +457,22 @@ class SimplePipeline(GObject.Object, Loggable):
 
         return position
 
-    @property
-    def _waiting_for_async_done(self):
-        return self.__waiting_for_async_done
-
-    @_waiting_for_async_done.setter
-    def _waiting_for_async_done(self, value):
-        self.__waiting_for_async_done = value
-
     def _recover(self):
-        if self._attempted_recoveries > MAX_RECOVERIES:
+        if not self._bus:
+            raise PipelineError("Should not try to recover after destroy")
+
+        if self._attempted_recoveries == MAX_RECOVERIES:
             self.emit("died")
-            self.error(
-                "Pipeline error detected multiple times in a row, not resetting anymore")
+            self.error("Declaring pipeline dead, because %d successive reset attempts failed", MAX_RECOVERIES)
             return
 
-        self.error("Pipeline error detected during playback, resetting"
-                   " -- num tries: %d", self._attempted_recoveries)
-
+        self._attempted_recoveries += 1
+        self.error("Resetting pipeline because error detected during playback. "
+                   "Try %d", self._attempted_recoveries)
         self.setState(Gst.State.NULL)
         self._recovery_state = self.RecoveryState.STARTED_RECOVERING
         self.pause()
 
-        self._attempted_recoveries += 1
-
     def _queryDurationAsync(self, *unused_args, **unused_kwargs):
         try:
             self.getDuration()
@@ -572,7 +481,7 @@ class SimplePipeline(GObject.Object, Loggable):
         return False
 
     def _handleErrorMessage(self, error, detail, source):
-        self.error("error from %s: %s (%s)" % (source, error, detail))
+        self.error("error from %s: %s (%s)", source, error, detail)
         self.emit('error', error.message, detail)
 
     def _getDuration(self):
@@ -591,33 +500,32 @@ class SimplePipeline(GObject.Object, Loggable):
 
 
 class AssetPipeline(SimplePipeline):
-
-    """
-    Pipeline for playing a single clip.
-    """
+    """Pipeline for playing a single clip."""
 
     def __init__(self, clip=None, name=None):
-        bPipeline = Gst.ElementFactory.make("playbin", name)
-        SimplePipeline.__init__(self, bPipeline)
+        ges_pipeline = Gst.ElementFactory.make("playbin", name)
+        SimplePipeline.__init__(self, ges_pipeline)
+
+        self.create_sink()
 
         self.clip = clip
         if self.clip:
             self.setClipUri(self.clip.props.uri)
 
+    def create_sink(self):
+        SimplePipeline.create_sink(self)
+        self._pipeline.set_property("video_sink", self.video_sink)
+
     def setClipUri(self, uri):
         self._pipeline.set_property("uri", uri)
 
 
 class Pipeline(GES.Pipeline, SimplePipeline):
-
-    """
-    Helper to handle GES.Pipeline through the SimplePipeline API
-    and handle the Seeker properly
-    """
+    """Helper to handle GES.Pipeline through the SimplePipeline API."""
 
     __gsignals__ = PIPELINE_SIGNALS
 
-    def __init__(self, app, pipeline=None):
+    def __init__(self, app):
         GES.Pipeline.__init__(self)
         SimplePipeline.__init__(self, self)
 
@@ -626,11 +534,6 @@ class Pipeline(GES.Pipeline, SimplePipeline):
         self._was_empty = False
         self._commit_wanted = False
 
-        self._timeline = None
-        self._seeker = Seeker()
-        self._seeker.connect("seek", self._seekCb)
-        self._seeker.connect("seek-relative", self._seekRelativeCb)
-
         if "watchdog" in os.environ.get("PITIVI_UNSTABLE_FEATURES", ''):
             watchdog = Gst.ElementFactory.make("watchdog", None)
             if watchdog:
@@ -640,39 +543,31 @@ class Pipeline(GES.Pipeline, SimplePipeline):
                 watchdog.props.timeout = WATCHDOG_TIMEOUT * 1000
                 self.props.audio_filter = watchdog
 
+    def create_sink(self):
+        SimplePipeline.create_sink(self)
+        self._pipeline.preview_set_video_sink(self.video_sink)
+
     def set_mode(self, mode):
         self._next_seek = None
         return GES.Pipeline.set_mode(self, mode)
 
     def _getDuration(self):
-        return self._timeline.get_duration()
+        return self.props.timeline.get_duration()
 
-    def set_timeline(self, timeline):
-        if not GES.Pipeline.set_timeline(self, timeline):
-            raise PipelineError("Cannot set the timeline to the pipeline")
-        self._timeline = timeline
+    def do_change_state(self, state):
+        if state == Gst.StateChange.PAUSED_TO_READY:
+            self._removeWaitingForAsyncDoneTimeout()
 
-    def release(self):
-        """
-        Release the L{Pipeline} and all used L{ObjectFactory} and
-        L{Action}s.
-
-        Call this method when the L{Pipeline} is no longer used. Forgetting to do
-        so will result in memory loss.
-
-        @postcondition: The L{Pipeline} will no longer be usable.
-        """
-        self._seeker.disconnect_by_func(self._seekRelativeCb)
-        self._seeker.disconnect_by_func(self._seekCb)
-        SimplePipeline.release(self)
-
-    def _seekRelativeCb(self, unused_seeker, time_delta):
-        self.seekRelative(time_delta)
+        return GES.Pipeline.do_change_state(self, state)
 
     def stepFrame(self, framerate, frames_offset):
-        """
-        Seek backwards or forwards a certain amount of frames (frames_offset).
+        """Seeks backwards or forwards the specified amount of frames.
+
         This clamps the playhead to the project frames.
+
+        Args:
+            frames_offsets (int): The number of frames to step. Negative number
+                for stepping backwards.
         """
         try:
             position = self.getPosition()
@@ -693,68 +588,64 @@ class Pipeline(GES.Pipeline, SimplePipeline):
                       new_pos / float(Gst.SECOND))
         self.simple_seek(new_pos)
 
-    def _seekCb(self, unused_seeker, position):
-        """
-        The app's main seek method used when the user seeks manually.
-
-        We clamp the seeker position so that it cannot go past 0 or the
-        end of the timeline.
-        """
-        self.simple_seek(position)
-
     def simple_seek(self, position):
-        if self._timeline.is_empty():
+        if self.props.timeline.is_empty():
+            # Nowhere to seek.
             return
 
         if self._rendering():
             raise PipelineError("Trying to seek while rendering")
 
         st = Gst.Structure.new_empty("seek")
-
         if self.getState() == Gst.State.PLAYING:
             st.set_value("playback_time", float(
                 self.getPosition()) / Gst.SECOND)
-
         st.set_value("start", float(position / Gst.SECOND))
         st.set_value("flags", "accurate+flush")
         self.app.write_action(st)
 
-        SimplePipeline.simple_seek(self, position)
+        try:
+            SimplePipeline.simple_seek(self, position)
+        except PipelineError as e:
+            self.error("Error while seeking to position: %s, reason: %s",
+                       format_ns(position), e)
 
     def _busMessageCb(self, bus, message):
         if message.type == Gst.MessageType.ASYNC_DONE:
-            self.commiting = False
+            self.app.gui.timeline_ui.timeline.update_visible_overlays()
+
         if message.type == Gst.MessageType.ASYNC_DONE and\
                 self._commit_wanted:
-            self.debug("Commiting now that ASYNC is DONE")
+            self.debug("Committing now that ASYNC is DONE")
             self._addWaitingForAsyncDoneTimeout()
-            self._timeline.commit()
+            self.props.timeline.commit()
             self._commit_wanted = False
         else:
-            super(Pipeline, self)._busMessageCb(bus, message)
+            SimplePipeline._busMessageCb(self, bus, message)
 
     def commit_timeline(self):
-        if self._waiting_for_async_done and not self._was_empty\
-                and not self._timeline.is_empty():
+        if self.getState() == Gst.State.NULL:
+            # No need to commit. NLE will do it automatically when
+            # changing state from READY to PAUSED.
+            return
+        is_empty = self.props.timeline.is_empty()
+        if self._busy_async and not self._was_empty and not is_empty:
             self._commit_wanted = True
             self._was_empty = False
-            self.debug("commit wanted")
+            self.log("commit wanted")
         else:
             self._addWaitingForAsyncDoneTimeout()
-            self._timeline.commit()
-            self.debug("Commiting right now")
-            if self._timeline.is_empty():
-                self._was_empty = True
-            else:
-                self._was_empty = False
+            self.props.timeline.commit()
+            self.debug("Committing right now")
+            self._was_empty = is_empty
 
     def setState(self, state):
-        super(Pipeline, self).setState(state)
-        if state >= Gst.State.PAUSED and self._timeline.is_empty():
-            self.debug("No ASYNC_DONE will be emited on empty timelines")
+        SimplePipeline.setState(self, state)
+        if state >= Gst.State.PAUSED and self.props.timeline.is_empty():
+            self.debug("No ASYNC_DONE will be emitted on empty timelines")
             self._was_empty = True
             self._removeWaitingForAsyncDoneTimeout()
 
     def _rendering(self):
         mask = GES.PipelineFlags.RENDER | GES.PipelineFlags.SMART_RENDER
-        return self._pipeline.get_mode() & mask
+        return self._pipeline.get_mode() & mask != 0
