@@ -50,6 +50,10 @@ GlobalSettings.addConfigOption('numTranscodingJobs',
                                section='proxy',
                                key='num-proxying-jobs',
                                default=4)
+GlobalSettings.addConfigOption("max_cpu_usage",
+                               section="proxy",
+                               key="max-cpu-usage",
+                               default=10)
 
 
 ENCODING_FORMAT_PRORES = "prores-opus-in-matroska.gep"
@@ -83,17 +87,26 @@ class ProxyManager(GObject.Object, Loggable):
                                                                   object)),
     }
 
+    WHITELIST_CONTAINER_CAPS = ["video/quicktime", "application/ogg",
+                                "video/x-matroska", "video/webm"]
+    WHITELIST_AUDIO_CAPS = ["audio/mpeg", "audio/x-vorbis",
+                            "audio/x-raw", "audio/x-flac",
+                            "audio/x-wav"]
+    WHITELIST_VIDEO_CAPS = ["video/x-h264", "image/jpeg",
+                            "video/x-raw", "video/x-vp8",
+                            "video/x-theora"]
+
     WHITELIST_FORMATS = []
-    for container in ["video/quicktime", "application/ogg",
-                      "video/x-matroska", "video/webm"]:
-        for audio in ["audio/mpeg", "audio/x-vorbis",
-                      "audio/x-raw", "audio/x-flac"]:
-            for video in ["video/x-h264", "image/jpeg",
-                          "video/x-raw", "video/x-vp8",
-                          "video/x-theora"]:
+    for container in WHITELIST_CONTAINER_CAPS:
+        for audio in WHITELIST_AUDIO_CAPS:
+            for video in WHITELIST_VIDEO_CAPS:
                 WHITELIST_FORMATS.append(createEncodingProfileSimple(
                     container, audio, video))
 
+    for audio in WHITELIST_AUDIO_CAPS:
+        a = GstPbutils.EncodingAudioProfile.new(Gst.Caps(audio), None, None, 0)
+        WHITELIST_FORMATS.append(a)
+
     proxy_extension = "proxy.mkv"
 
     def __init__(self, app):
@@ -101,10 +114,11 @@ class ProxyManager(GObject.Object, Loggable):
         Loggable.__init__(self)
 
         self.app = app
+        # Total time to transcode in seconds.
         self._total_time_to_transcode = 0
-        self._total_transcoded_time = 0
+        # Transcoded time per asset in seconds.
+        self._transcoded_durations = {}
         self._start_proxying_time = 0
-        self._estimated_time = 0
         self.__running_transcoders = []
         self.__pending_transcoders = []
 
@@ -128,6 +142,16 @@ class ProxyManager(GObject.Object, Loggable):
             return not info.get_caps().intersect(profile.get_format()).is_empty()
 
         info = asset.get_info()
+        if isinstance(encoding_profile, GstPbutils.EncodingAudioProfile):
+            if isinstance(info.get_stream_info(), GstPbutils.DiscovererContainerInfo):
+                return False
+            audios = info.get_audio_streams()
+            if len(audios) != 1 or not capsMatch(audios[0], encoding_profile):
+                return False
+            if info.get_video_streams():
+                return False
+            return True
+
         container = info.get_stream_info()
         if container:
             if not capsMatch(container, encoding_profile):
@@ -212,9 +236,15 @@ class ProxyManager(GObject.Object, Loggable):
             <filename>.<file_size>.<proxy_extension>
         """
         asset_file = Gio.File.new_for_uri(asset.get_id())
-        file_size = asset_file.query_info(Gio.FILE_ATTRIBUTE_STANDARD_SIZE,
-                                          Gio.FileQueryInfoFlags.NONE,
-                                          None).get_size()
+        try:
+            file_size = asset_file.query_info(Gio.FILE_ATTRIBUTE_STANDARD_SIZE,
+                                              Gio.FileQueryInfoFlags.NONE,
+                                              None).get_size()
+        except GLib.Error as err:
+            if err.matches(Gio.io_error_quark(), Gio.IOErrorEnum.NOT_FOUND):
+                return None
+            else:
+                raise
 
         return "%s.%s.%s" % (asset.get_id(), file_size, self.proxy_extension)
 
@@ -226,7 +256,7 @@ class ProxyManager(GObject.Object, Loggable):
 
         return False
 
-    def __assetNeedsTranscoding(self, asset, force_proxying=False):
+    def __assetNeedsTranscoding(self, asset):
         if self.proxyingUnsupported:
             self.info("No proxying supported")
             return False
@@ -234,10 +264,6 @@ class ProxyManager(GObject.Object, Loggable):
         if asset.is_image():
             return False
 
-        if force_proxying:
-            self.info("Forcing proxy creation")
-            return True
-
         if self.app.settings.proxyingStrategy == ProxyingStrategy.NOTHING:
             self.debug("Not proxying anything. %s",
                        self.app.settings.proxyingStrategy)
@@ -295,7 +321,7 @@ class ProxyManager(GObject.Object, Loggable):
         self.emit("proxy-ready", asset, proxy)
         self.__emitProgress(proxy, 100)
 
-    def __transcoderErrorCb(self, transcoder, error, asset):
+    def __transcoderErrorCb(self, transcoder, error, unused_details, asset):
         self.emit("error-preparing-asset", asset, None, error)
 
     def __transcoderDoneCb(self, transcoder, asset):
@@ -321,32 +347,37 @@ class ProxyManager(GObject.Object, Loggable):
             self.__startTranscoder(self.__pending_transcoders.pop())
         except IndexError:
             if not self.__running_transcoders:
-                self._total_transcoded_time = 0
+                self._transcoded_durations = {}
                 self._total_time_to_transcode = 0
                 self._start_proxying_time = 0
 
-    def __emitProgress(self, asset, progress):
-        if self._total_transcoded_time:
+    def __emitProgress(self, asset, creation_progress):
+        """Handles the transcoding progress of the specified asset."""
+        if self._transcoded_durations:
             time_spent = time.time() - self._start_proxying_time
-            self._estimated_time = max(
-                0, (time_spent * self._total_time_to_transcode /
-                    self._total_transcoded_time) - time_spent)
+            transcoded_seconds = sum(self._transcoded_durations.values())
+            remaining_seconds = max(0, self._total_time_to_transcode - transcoded_seconds)
+            estimated_time = remaining_seconds * time_spent / transcoded_seconds
         else:
-            self._estimated_time = 0
+            estimated_time = 0
 
-        asset.creation_progress = progress
-        self.emit("progress", asset, asset.creation_progress,
-                  self._estimated_time)
+        asset.creation_progress = creation_progress
+        self.emit("progress", asset, asset.creation_progress, estimated_time)
 
     def __proxyingPositionChangedCb(self, transcoder, position, asset):
-        # Do not set to >= 100 as we need to notify about the proxy first
-        self._total_transcoded_time -= (asset.creation_progress * (asset.get_duration() /
-                                                                   Gst.SECOND)) / 100
-        self._total_transcoded_time += position / Gst.SECOND
+        if transcoder not in self.__running_transcoders:
+            self.info("Position changed after job cancelled!")
+            return
 
-        if transcoder.props.duration:
-            asset.creation_progress = max(
-                0, min(99, (position / transcoder.props.duration) * 100))
+        self._transcoded_durations[asset] = position / Gst.SECOND
+
+        duration = transcoder.props.duration
+        if duration <= 0 or duration == Gst.CLOCK_TIME_NONE:
+            duration = asset.props.duration
+        if duration > 0 and duration != Gst.CLOCK_TIME_NONE:
+            creation_progress = 100 * position / duration
+            # Do not set to >= 100 as we need to notify about the proxy first.
+            asset.creation_progress = max(0, min(creation_progress, 99))
 
         self.__emitProgress(asset, asset.creation_progress)
 
@@ -388,7 +419,7 @@ class ProxyManager(GObject.Object, Loggable):
         transcoder.props.pipeline.props.video_filter = thumbnailbin
         transcoder.props.pipeline.props.audio_filter = waveformbin
 
-        transcoder.set_cpu_usage(10)
+        transcoder.set_cpu_usage(self.app.settings.max_cpu_usage)
         transcoder.connect("position-updated",
                            self.__proxyingPositionChangedCb,
                            asset)
@@ -400,59 +431,80 @@ class ProxyManager(GObject.Object, Loggable):
         else:
             self.__pending_transcoders.append(transcoder)
 
-    def cancelJob(self, asset):
+    def cancel_job(self, asset):
+        """Cancels the transcoding job for the specified asset, if any.
+
+        Args:
+            asset (GES.Asset): The original asset.
+        """
         if not self.is_asset_queued(asset):
             return
 
         for transcoder in self.__running_transcoders:
             if asset.props.id == transcoder.props.src_uri:
-                self.__running_transcoders.remove(transcoder)
                 self.info("Cancelling running transcoder %s %s",
                           transcoder.props.src_uri,
                           transcoder.__grefcount__)
+                self.__running_transcoders.remove(transcoder)
                 self.emit("asset-preparing-cancelled", asset)
                 return
 
         for transcoder in self.__pending_transcoders:
             if asset.props.id == transcoder.props.src_uri:
+                self.info("Cancelling pending transcoder %s",
+                          transcoder.props.src_uri)
                 # Removing the transcoder from the list
                 # will lead to its destruction (only reference)
                 # here, which means it will be stopped.
                 self.__pending_transcoders.remove(transcoder)
                 self.emit("asset-preparing-cancelled", asset)
-                self.info("Cancelling pending transcoder %s",
-                          transcoder.props.src_uri)
                 return
 
-        return
-
-    def addJob(self, asset, force_proxying=False):
-        self.debug("Maybe create a proxy for %s (strategy: %s)",
-                   asset.get_id(), self.app.settings.proxyingStrategy)
-
-        if not self.__assetNeedsTranscoding(asset, force_proxying):
-            self.debug("Not proxying asset (settings.proxyingStrategy: %s,"
-                       " proxy support forced: %s disabled: %s)",
-                       self.app.settings.proxyingStrategy,
-                       force_proxying, self.proxyingUnsupported)
-
-            # Make sure to notify we do not need a proxy for
-            # that asset.
-            self.emit("proxy-ready", asset, None)
-            return True
+    def add_job(self, asset):
+        """Adds a transcoding job for the specified asset if needed.
 
+        Args:
+            asset (GES.Asset): The asset to be transcoded.
+        """
         if self.is_asset_queued(asset):
-            return True
+            self.log("Asset already queued for proxying: %s", asset)
+            return
+
+        force_proxying = asset.force_proxying
+        if not force_proxying and not self.__assetNeedsTranscoding(asset):
+            self.debug("Not proxying asset (proxying disabled: %s)",
+                       self.proxyingUnsupported)
+            # Make sure to notify we do not need a proxy for that asset.
+            self.emit("proxy-ready", asset, None)
+            return
 
         proxy_uri = self.getProxyUri(asset)
         if Gio.File.new_for_uri(proxy_uri).query_exists(None):
-            self.debug("Using proxy already generated: %s",
-                       proxy_uri)
+            self.debug("Using proxy already generated: %s", proxy_uri)
             GES.Asset.request_async(GES.UriClip,
                                     proxy_uri, None,
                                     self.__assetLoadedCb, asset,
                                     None)
-            return True
+            return
 
+        self.debug("Creating a proxy for %s (strategy: %s, force: %s)",
+                   asset.get_id(), self.app.settings.proxyingStrategy,
+                   force_proxying)
         self.__createTranscoder(asset)
-        return True
+        return
+
+
+def get_proxy_target(obj):
+    if isinstance(obj, GES.UriClip):
+        asset = obj.get_asset()
+    elif isinstance(obj, GES.TrackElement):
+        asset = obj.get_parent().get_asset()
+    else:
+        asset = obj
+
+    if ProxyManager.is_proxy_asset(asset):
+        target = asset.get_proxy_target()
+        if target and target.get_error() is None:
+            asset = target
+
+    return asset
