@@ -66,8 +66,10 @@ class Selected(GObject.Object):
 
     @selected.setter
     def selected(self, selected):
+        differs = self._selected != selected
         self._selected = selected
-        self.emit("selected-changed", selected)
+        if differs:
+            self.emit("selected-changed", selected)
 
 
 class Selection(GObject.Object, Loggable):
@@ -88,6 +90,8 @@ class Selection(GObject.Object, Loggable):
         GObject.Object.__init__(self)
         Loggable.__init__(self)
         self.selected = set()
+        self.can_group = False
+        self.can_ungroup = False
 
     def setSelection(self, objs, mode):
         """Updates the current selection.
@@ -133,9 +137,23 @@ class Selection(GObject.Object, Loggable):
                         isinstance(obj, GES.TextOverlay):
                     continue
                 element.selected.selected = selected
-
+        self.set_can_group_ungroup()
         self.emit("selection-changed")
 
+    def set_can_group_ungroup(self):
+        containers = set()
+        for obj in self.selected:
+            toplevel = obj.get_toplevel_parent()
+            if not toplevel.props.serialize:
+                for child in toplevel.get_children(False):
+                    containers.add(child)
+            else:
+                containers.add(toplevel)
+            if len(containers) > 1:
+                break
+        self.can_group = len(containers) > 1
+        self.can_ungroup = len(self.getSelectedTrackElements()) > 1
+
     def __get_selection_changes(self, old_selection):
         for obj in old_selection - self.selected:
             yield obj, False
@@ -220,6 +238,8 @@ class EditingContext(GObject.Object, Loggable):
         else:
             self.focus = focus
 
+        self.with_video = GES.TrackType.VIDEO & self.focus.get_track_types()
+
         self.old_position = self.focus.get_start()
         if edge == GES.Edge.EDGE_END and mode == GES.EditMode.EDIT_TRIM:
             self.old_position += self.focus.get_duration()
@@ -280,7 +300,7 @@ class EditingContext(GObject.Object, Loggable):
             edge=self.edge.value_nick,
             new_layer_priority=int(priority))
 
-        if res and self.mode == GES.EditMode.EDIT_TRIM:
+        if res and self.mode == GES.EditMode.EDIT_TRIM and self.with_video:
             if self.edge == GES.Edge.EDGE_START:
                 self.timeline.ui.app.gui.viewer.clipTrimPreview(self.focus, self.focus.props.in_point)
             elif self.edge == GES.Edge.EDGE_END:
