@@ -22,10 +22,15 @@ A collection of objects to use for testing
 import contextlib
 import gc
 import os
+import shutil
+import sys
 import tempfile
+import traceback
 import unittest
 from unittest import mock
 
+from gi.repository import Gdk
+from gi.repository import GES
 from gi.repository import GLib
 from gi.repository import Gst
 from gi.repository import Gtk
@@ -34,19 +39,37 @@ from pitivi import check
 from pitivi.application import Pitivi
 from pitivi.project import ProjectManager
 from pitivi.settings import GlobalSettings
+from pitivi.timeline.previewers import Previewer
+from pitivi.timeline.previewers import PreviewGeneratorManager
 from pitivi.timeline.timeline import TimelineContainer
 from pitivi.utils.loggable import Loggable
+from pitivi.utils.misc import path_from_uri
 from pitivi.utils.proxy import ProxyingStrategy
 from pitivi.utils.proxy import ProxyManager
 from pitivi.utils.timeline import Selected
+from pitivi.utils.timeline import Zoomable
+
+
+def handle_uncaught_exception(exctype, value, trace):
+    traceback.print_tb(trace)
+    print(value, file=sys.stderr)
+    sys.exit(1)
+
+
+sys.excepthook = handle_uncaught_exception
+
+
+def handle_glog(domain, level, message, udata):
+    traceback.print_stack()
+    print("%s - %s" % (domain, message), file=sys.stderr)
+    sys.exit(1)
+
+
+for category in ["GStreamer", "Gtk", "Gdk", "GLib-GObject", "GES"]:
+    GLib.log_set_handler(category, GLib.LogLevelFlags.LEVEL_CRITICAL, handle_glog, None)
 
 detect_leaks = os.environ.get("PITIVI_TEST_DETECT_LEAKS", "0") not in ("0", "")
-os.environ["PITIVI_USER_CACHE_DIR"] = tempfile.mkdtemp("pitiviTestsuite")
-
-
-def clean_pitivi_mock(app):
-    app.settings = None
-    app.proxy_manager = None
+os.environ["PITIVI_USER_CACHE_DIR"] = tempfile.mkdtemp(suffix="pitiviTestsuite")
 
 
 def __create_settings(proxyingStrategy=ProxyingStrategy.NOTHING,
@@ -69,6 +92,10 @@ def create_pitivi_mock(**settings):
     app.settings = __create_settings(**settings)
     app.proxy_manager = ProxyManager(app)
 
+    # TODO: Get rid of Zoomable.app.
+    from pitivi.utils.timeline import Zoomable
+    Zoomable.app = app
+
     return app
 
 
@@ -88,7 +115,7 @@ def create_pitivi(**settings):
 
 
 def create_timeline_container():
-    app = create_pitivi_mock()
+    app = create_pitivi_mock(leftClickAlsoSeeks=False)
     project_manager = ProjectManager(app)
     project_manager.newBlankProject()
     project = project_manager.current_project
@@ -101,7 +128,6 @@ def create_timeline_container():
     timeline.get_parent = mock.MagicMock(return_value=timeline_container)
 
     app.gui.timeline_ui = timeline_container
-    timeline.app.settings.leftClickAlsoSeeks = False
 
     return timeline_container
 
@@ -110,15 +136,17 @@ def create_main_loop():
     mainloop = GLib.MainLoop()
     timed_out = False
 
-    def quit_cb(unused):
+    def timeout_cb(unused):
         nonlocal timed_out
         timed_out = True
         mainloop.quit()
 
-    def run(timeout_seconds=5):
+    def run(timeout_seconds=5, until_empty=False):
         source = GLib.timeout_source_new_seconds(timeout_seconds)
-        source.set_callback(quit_cb)
+        source.set_callback(timeout_cb)
         source.attach()
+        if until_empty:
+            GLib.idle_add(mainloop.quit)
         GLib.MainLoop.run(mainloop)
         source.destroy()
         if timed_out:
@@ -175,11 +203,21 @@ class TestCase(unittest.TestCase, Loggable):
         del self._tracked
 
     def setUp(self):
+        # TODO: Get rid of Zoomable._instances.
+        from pitivi.utils.timeline import Zoomable
+        del Zoomable._instances[:]
+
         self._num_failures = len(getattr(self._result, 'failures', []))
         self._num_errors = len(getattr(self._result, 'errors', []))
         if detect_leaks:
             self.gctrack()
 
+        self.__zoom_level = Zoomable.getCurrentZoomLevel()
+
+        # TODO: Get rid of Previewer.manager.
+        assert hasattr(Previewer, "manager")
+        Previewer.manager = PreviewGeneratorManager()
+
     def tearDown(self):
         # don't barf gc info all over the console if we have already failed a
         # test case
@@ -189,6 +227,7 @@ class TestCase(unittest.TestCase, Loggable):
         if detect_leaks:
             self.gccollect()
             self.gcverify()
+        Zoomable.setZoomLevel(self.__zoom_level)
 
     # override run() to save a reference to the test result object
     def run(self, result=None):
@@ -203,22 +242,36 @@ class TestCase(unittest.TestCase, Loggable):
         self.assertEqual(ges_clip.selected.selected, selected)
 
         # Simulate a click on the clip.
-        event = mock.Mock()
+        event = mock.Mock(spec=Gdk.EventButton)
+        event.x = 0
+        event.y = 0
         event.get_button.return_value = (True, 1)
         with mock.patch.object(Gtk, "get_event_widget") as get_event_widget:
             get_event_widget.return_value = ges_clip.ui
             ges_clip.ui.timeline._button_press_event_cb(None, event)
-        ges_clip.ui._button_release_event_cb(None, event)
+            with mock.patch.object(ges_clip.ui, "translate_coordinates") as translate_coordinates:
+                translate_coordinates.return_value = (0, 0)
+                with mock.patch.object(ges_clip.ui.timeline, "_get_layer_at") as _get_layer_at:
+                    _get_layer_at.return_value = ges_clip.props.layer, None
+                    ges_clip.ui._button_release_event_cb(None, event)
 
         self.assertEqual(bool(ges_clip.ui.get_state_flags() & Gtk.StateFlags.SELECTED),
                          expect_selected)
         self.assertEqual(ges_clip.selected.selected, expect_selected)
 
+    def assert_caps_equal(self, caps1, caps2):
+        if isinstance(caps1, str):
+            caps1 = Gst.Caps(caps1)
+        if isinstance(caps2, str):
+            caps2 = Gst.Caps(caps2)
+
+        self.assertTrue(caps1.is_equal(caps2),
+                        "%s != %s" % (caps1.to_string(), caps2.to_string()))
+
 
 @contextlib.contextmanager
-def created_project_file(asset_uri="file:///icantpossiblyexist.png"):
-    """
-    Create a project file.
+def created_project_file(asset_uri):
+    """Creates a project file.
 
     Yields:
         str: The URI of the new project
@@ -247,19 +300,41 @@ def created_project_file(asset_uri="file:///icantpossiblyexist.png"):
     os.remove(xges_path)
 
 
-def get_sample_uri(sample):
-    tests_dir = os.path.dirname(os.path.abspath(__file__))
-    return Gst.filename_to_uri(os.path.join(tests_dir, "samples", sample))
+def get_sample_uri(sample, samples_dir=None):
+    if not samples_dir:
+        tests_dir = os.path.dirname(os.path.abspath(__file__))
+        samples_dir = os.path.join(tests_dir, "samples")
+    return Gst.filename_to_uri(os.path.join(samples_dir, sample))
 
 
-def clean_proxy_samples():
-    _dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "samples")
-    proxy_manager = ProxyManager(mock.MagicMock())
+def get_sample_clip(sample):
+    uri = get_sample_uri(sample)
+    asset = GES.UriClipAsset.request_sync(uri)
+    clip = asset.extract()
+    return clip
 
-    for f in os.listdir(_dir):
-        if f.endswith(proxy_manager.proxy_extension):
-            f = os.path.join(_dir, f)
-            os.remove(f)
+
+@contextlib.contextmanager
+def cloned_sample(*samples):
+    """Gets a context manager which commits the transaction at the end."""
+    with tempfile.TemporaryDirectory() as tmpdir:
+        module = globals()
+        original_get_sample_uri = module["get_sample_uri"]
+        module["get_sample_uri"] = lambda sample: original_get_sample_uri(sample, samples_dir=tmpdir)
+        try:
+            for sample in samples:
+                sample_path = path_from_uri(original_get_sample_uri(sample))
+                clone_path = path_from_uri(get_sample_uri(sample))
+                shutil.copyfile(sample_path, clone_path)
+            yield tmpdir
+        finally:
+            module["get_sample_uri"] = original_get_sample_uri
+
+
+def get_clip_children(ges_clip, *track_types, recursive=False):
+    for ges_timeline_element in ges_clip.get_children(recursive):
+        if not track_types or ges_timeline_element.get_track_type() in track_types:
+            yield ges_timeline_element
 
 
 def create_test_clip(clip_type):
