@@ -4,22 +4,17 @@
 # Flumotion - a streaming media server
 # Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
-
 # This file may be distributed and/or modified under the terms of
 # the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 # This file is distributed without any warranty; without even the implied
 # warranty of merchantability or fitness for a particular purpose.
 # See "LICENSE.GPL" in the source distribution for more information.
-
 # Licensees having purchased or holding a valid Flumotion Advanced
 # Streaming Server license may use this file in accordance with the
 # Flumotion Advanced Streaming Server Commercial License Agreement.
 # See "LICENSE.Flumotion" in the source distribution for more information.
-
 # Headers in this file shall remain intact.
-
-
 import unittest
 
 from pitivi.utils import loggable as log
@@ -214,49 +209,6 @@ class TestLogSettings(unittest.TestCase):
         self.assertEqual(old, log.getLogSettings())
 
 
-class TestWriteMark(TestWithHandler):
-
-    def testWriteMarkInDebug(self):
-        loggable = log.Loggable()
-        log.setDebug("%d" % log.DEBUG)
-        log.addLogHandler(self.handler)
-        marker = 'test'
-        loggable.writeMarker(marker, log.DEBUG)
-        self.assertEqual(self.message, marker)
-
-    def testWriteMarkInWarn(self):
-        loggable = log.Loggable()
-        log.setDebug("%d" % log.WARN)
-        log.addLogHandler(self.handler)
-        marker = 'test'
-        loggable.writeMarker(marker, log.WARN)
-        self.assertEqual(self.message, marker)
-
-    def testWriteMarkInInfo(self):
-        loggable = log.Loggable()
-        log.setDebug("%d" % log.INFO)
-        log.addLogHandler(self.handler)
-        marker = 'test'
-        loggable.writeMarker(marker, log.INFO)
-        self.assertEqual(self.message, marker)
-
-    def testWriteMarkInLog(self):
-        loggable = log.Loggable()
-        log.setDebug("%d" % log.LOG)
-        log.addLogHandler(self.handler)
-        marker = 'test'
-        loggable.writeMarker(marker, log.LOG)
-        self.assertEqual(self.message, marker)
-
-    def testWriteMarkInError(self):
-        loggable = log.Loggable()
-        log.setDebug("%d" % log.ERROR)
-        log.addLogHandler(self.handler)
-        marker = 'test'
-        loggable.writeMarker(marker, log.ERROR)
-        self.assertEqual(self.message, marker)
-
-
 class TestLogNames(unittest.TestCase):
 
     def testGetLevelNames(self):
