@@ -17,7 +17,7 @@
 # Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 # Boston, MA 02110-1301, USA.
 """
-Test for pitivi/mainwindow.py
+Tests for pitivi/mainwindow.py
 """
 from unittest import mock
 
@@ -30,15 +30,12 @@ from pitivi.utils.misc import disconnectAllByFunc
 from tests import common
 
 
+# pylint: disable=protected-access
 class TestMainWindow(common.TestCase):
-    """
-    Test MainWindow
-    """
+    """Tests for the MainWindow class."""
 
-    def testSwitchContextTab(self):
-        """
-        Test tab switches
-        """
+    def test_switch_context_tab(self):
+        """Checks tab switches."""
         app = common.create_pitivi_mock()
         mainwindow = MainWindow(app)
         for expected_tab, b_element in [
@@ -46,13 +43,13 @@ class TestMainWindow(common.TestCase):
                 (0, GES.SourceClip()),
                 (1, GES.TransitionClip())]:
             mainwindow.switchContextTab(b_element)
-            self.assertEqual(expected_tab,
-                             mainwindow.context_tabs.get_current_page(),
+            self.assertEqual(mainwindow.context_tabs.get_current_page(),
+                             expected_tab,
                              b_element)
             # Make sure the tab does not change when using an invalid argument.
             mainwindow.switchContextTab("invalid")
-            self.assertEqual(
-                expected_tab, mainwindow.context_tabs.get_current_page())
+            self.assertEqual(mainwindow.context_tabs.get_current_page(),
+                             expected_tab)
 
         mainwindow.destroy()
 
@@ -64,9 +61,10 @@ class TestMainWindow(common.TestCase):
         app.project_manager = ProjectManager(app)
         mainwindow = MainWindow(app)
         mainwindow.viewer = mock.MagicMock()
+        mainwindow.medialibrary._import_warning_infobar = mock.MagicMock()
+        mainwindow.clipconfig.effect_expander._infobar = mock.MagicMock()
 
-        def __pm_missing_uri_cb(project_manager, project,
-                                error, asset):
+        def __pm_missing_uri_cb(project_manager, project, error, asset):
             nonlocal mainloop
             nonlocal mainwindow
             nonlocal self
@@ -82,7 +80,6 @@ class TestMainWindow(common.TestCase):
                     return_value=Gtk.ResponseType.CLOSE)
 
                 # Call the actual callback
-                # pylint: disable=protected-access
                 app.proxy_manager.checkProxyLoadingSucceeded =  \
                     mock.MagicMock(return_value=has_proxy)
 
@@ -93,38 +90,32 @@ class TestMainWindow(common.TestCase):
                 self.assertTrue(dialog.return_value.run.called)
                 self.assertEqual(failed_cb.called, not has_proxy)
 
-            # pylint: disable=protected-access
             app.project_manager.connect("missing-uri",
                                         mainwindow._projectManagerMissingUriCb)
-            # pylint: disable=protected-access
             app.project_manager.connect("new-project-failed",
                                         mainwindow._projectManagerNewProjectFailedCb)
 
             mainwindow.destroy()
             mainloop.quit()
 
-        # pylint: disable=protected-access
         disconnectAllByFunc(app.project_manager,
                             mainwindow._projectManagerMissingUriCb)
         disconnectAllByFunc(app.project_manager,
                             mainwindow._projectManagerNewProjectFailedCb)
 
-        app.project_manager.connect("missing-uri",
-                                    __pm_missing_uri_cb)
+        app.project_manager.connect("missing-uri", __pm_missing_uri_cb)
 
-        with common.created_project_file() as uri:
-            app.project_manager.loadProject(uri)
+        with common.cloned_sample():
+            asset_uri = common.get_sample_uri("missing.png")
+            with common.created_project_file(asset_uri) as uri:
+                app.project_manager.loadProject(uri)
 
         mainloop.run()
 
     def test_loading_project_no_proxy(self):
-        """
-        Test loading failure without proxies
-        """
+        """Checks loading failure without proxies."""
         self.__loading_failure(has_proxy=False)
 
     def test_loading_project_with_proxy(self):
-        """
-        Test loading failure with proxies
-        """
+        """Checks loading failure with proxies."""
         self.__loading_failure(has_proxy=True)
