@@ -22,7 +22,6 @@ import os
 import tempfile
 import time
 from unittest import mock
-from unittest import TestCase
 
 from gi.repository import GES
 from gi.repository import Gst
@@ -59,10 +58,15 @@ class ProjectManagerListener(object):
         return True
 
 
-class TestProjectManager(TestCase):
+class TestProjectManager(common.TestCase):
 
     def setUp(self):
-        app = mock.MagicMock()
+        super(TestProjectManager, self).setUp()
+        self.setupApp()
+
+    def setupApp(self, app=None):
+        if not app:
+            app = mock.MagicMock()
         self.manager = ProjectManager(app)
         self.listener = ProjectManagerListener(self.manager)
         self.signals = self.listener.signals
@@ -87,7 +91,7 @@ class TestProjectManager(TestCase):
 
     def testLoadProjectClosesCurrent(self):
         """
-        Check that new-project-failed is emited if we can't close the current
+        Check that new-project-failed is emitted if we can't close the current
         project instance.
         """
         state = {"tried-close": False}
@@ -117,6 +121,7 @@ class TestProjectManager(TestCase):
         self.assertEqual("new-project-loaded", name, self.signals)
 
     def testMissingUriForwarded(self):
+        self.setupApp(app=common.create_pitivi_mock())
         mainloop = common.create_main_loop()
 
         def missingUriCb(self, project, error, clip_asset, result):
@@ -126,9 +131,11 @@ class TestProjectManager(TestCase):
         result = [False]
         self.manager.connect("missing-uri", missingUriCb, result)
 
-        with common.created_project_file() as uri:
-            self.assertTrue(self.manager.loadProject(uri))
-            mainloop.run()
+        with common.cloned_sample():
+            asset_uri = common.get_sample_uri("missing.png")
+            with common.created_project_file(asset_uri) as uri:
+                self.assertTrue(self.manager.loadProject(uri))
+                mainloop.run()
         self.assertTrue(result[0], "missing-uri has not been emitted")
 
     def testLoaded(self):
@@ -139,10 +146,11 @@ class TestProjectManager(TestCase):
 
         self.manager.connect("new-project-loaded", new_project_loaded_cb)
 
-        asset_uri = common.get_sample_uri("flat_colour1_640x480.png")
-        with common.created_project_file(asset_uri=asset_uri) as uri:
-            self.assertTrue(self.manager.loadProject(uri))
-            mainloop.run()
+        with common.cloned_sample("flat_colour1_640x480.png"):
+            asset_uri = common.get_sample_uri("flat_colour1_640x480.png")
+            with common.created_project_file(asset_uri=asset_uri) as uri:
+                self.assertTrue(self.manager.loadProject(uri))
+                mainloop.run()
 
         project = self.manager.current_project
         self.assertFalse(project.at_least_one_asset_missing)
@@ -344,6 +352,16 @@ class TestProjectLoading(common.TestCase):
 
         self.assertEqual(len(assets), 1, assets)
 
+    def create_project_file_from_xges(self, app, xges):
+        unused, xges_path = tempfile.mkstemp(suffix=".xges")
+        proj_uri = Gst.filename_to_uri(os.path.abspath(xges_path))
+        app.project_manager.saveProject(uri=proj_uri)
+
+        with open(xges_path, "w") as f:
+            f.write(xges)
+
+        return proj_uri
+
     def load_project_with_missing_proxy(self):
         """Loads a project with missing proxies."""
         uris = [common.get_sample_uri("1sec_simpsons_trailer.mp4")]
@@ -377,19 +395,11 @@ class TestProjectLoading(common.TestCase):
         app = common.create_pitivi(proxyingStrategy=ProxyingStrategy.ALL)
         proxy_manager = app.proxy_manager
         project_manager = app.project_manager
+        medialib = medialibrary.MediaLibraryWidget(app)
 
         mainloop = common.create_main_loop()
 
-        unused, xges_path = tempfile.mkstemp(suffix=".xges")
-        proj_uri = "file://" + os.path.abspath(xges_path)
-        app.project_manager.saveProject(uri=proj_uri)
-        medialib = medialibrary.MediaLibraryWidget(app)
-
-        with open(proj_uri[len("file://"):], "w") as f:
-            f.write(PROJECT_STR)
-
-        # Remove proxy
-        common.clean_proxy_samples()
+        proj_uri = self.create_project_file_from_xges(app, PROJECT_STR)
 
         def closing_project_cb(*args, **kwargs):
             # Do not ask whether to save project on closing.
@@ -407,8 +417,10 @@ class TestProjectLoading(common.TestCase):
 
     def test_load_project_with_missing_proxy(self):
         """Checks loading a project with missing proxies."""
-        mainloop, app, medialib, proxy_uri = self.load_project_with_missing_proxy()
-        mainloop.run()
+        with common.cloned_sample("1sec_simpsons_trailer.mp4"):
+            mainloop, app, medialib, proxy_uri = self.load_project_with_missing_proxy()
+            mainloop.run()
+
         self.assertEqual(len(medialib.storemodel), 1)
         self.assertEqual(medialib.storemodel[0][medialibrary.COL_ASSET].props.id,
                          proxy_uri)
@@ -419,49 +431,51 @@ class TestProjectLoading(common.TestCase):
         """Checks progress tracking of loading project with missing proxies."""
         from gi.repository import GstTranscoder
 
-        # Disable proxy generation by not making it start ever.
-        # This way we are sure it will not finish before we test
-        # the state while it is being rebuilt.
-        with mock.patch.object(GstTranscoder.Transcoder, "run_async"):
-            mainloop, app, medialib, proxy_uri = self.load_project_with_missing_proxy()
-            uri = common.get_sample_uri("1sec_simpsons_trailer.mp4")
+        with common.cloned_sample("1sec_simpsons_trailer.mp4"):
+            # Disable proxy generation by not making it start ever.
+            # This way we are sure it will not finish before we test
+            # the state while it is being rebuilt.
+            with mock.patch.object(GstTranscoder.Transcoder, "run_async"):
+                mainloop, app, medialib, proxy_uri = self.load_project_with_missing_proxy()
+                uri = common.get_sample_uri("1sec_simpsons_trailer.mp4")
 
-            app.project_manager.connect("new-project-loaded", lambda x, y: mainloop.quit())
-            mainloop.run()
+                app.project_manager.connect("new-project-loaded", lambda x, y: mainloop.quit())
+                mainloop.run()
 
-            self.assertEqual(len(medialib.storemodel), 1)
-            self.assertEqual(medialib.storemodel[0][medialibrary.COL_ASSET].props.id,
-                             uri)
-            self.assertEqual(medialib.storemodel[0][medialibrary.COL_THUMB_DECORATOR].state,
-                             medialibrary.AssetThumbnail.IN_PROGRESS)
+        self.assertEqual(len(medialib.storemodel), 1)
+        self.assertEqual(medialib.storemodel[0][medialibrary.COL_ASSET].props.id,
+                         uri)
+        self.assertEqual(medialib.storemodel[0][medialibrary.COL_THUMB_DECORATOR].state,
+                         medialibrary.AssetThumbnail.IN_PROGRESS)
 
     def test_load_project_with_missing_proxy_stop_generating_and_proxy(self):
         """Checks cancelling creation of a missing proxies and forcing it again."""
         from gi.repository import GstTranscoder
 
-        # Disable proxy generation by not making it start ever.
-        # This way we are sure it will not finish before we test
-        # stop generating the proxy and restart it.
-        with mock.patch.object(GstTranscoder.Transcoder, "run_async"):
-            mainloop, app, medialib, proxy_uri = self.load_project_with_missing_proxy()
-            uri = common.get_sample_uri("1sec_simpsons_trailer.mp4")
+        with common.cloned_sample("1sec_simpsons_trailer.mp4"):
+            # Disable proxy generation by not making it start ever.
+            # This way we are sure it will not finish before we test
+            # stop generating the proxy and restart it.
+            with mock.patch.object(GstTranscoder.Transcoder, "run_async"):
+                mainloop, app, medialib, proxy_uri = self.load_project_with_missing_proxy()
 
-            app.project_manager.connect("new-project-loaded", lambda x, y: mainloop.quit())
-            mainloop.run()
-            asset = medialib.storemodel[0][medialibrary.COL_ASSET]
-            app.project_manager.current_project.disable_proxies_for_assets([asset])
+                app.project_manager.connect("new-project-loaded", lambda x, y: mainloop.quit())
+                mainloop.run()
+                asset = medialib.storemodel[0][medialibrary.COL_ASSET]
+                app.project_manager.current_project.disable_proxies_for_assets([asset])
 
             row, = medialib.storemodel
             asset = row[medialibrary.COL_ASSET]
             self.assertEqual(medialib._progressbar.get_fraction(), 1.0)
+            uri = common.get_sample_uri("1sec_simpsons_trailer.mp4")
             self.assertEqual(asset.props.id, uri)
             self.assertEqual(asset.ready, True)
             self.assertEqual(asset.creation_progress, 100)
             self.assertEqual(row[medialibrary.COL_THUMB_DECORATOR].state,
                              medialibrary.AssetThumbnail.NO_PROXY)
 
-        app.project_manager.current_project.use_proxies_for_assets([asset])
-        mainloop.run()
+            app.project_manager.current_project.use_proxies_for_assets([asset])
+            mainloop.run()
 
         row, = medialib.storemodel
         asset = row[medialibrary.COL_ASSET]
@@ -472,6 +486,88 @@ class TestProjectLoading(common.TestCase):
         self.assertEqual(row[medialibrary.COL_THUMB_DECORATOR].state,
                          medialibrary.AssetThumbnail.PROXIED)
 
+    def test_loading_project_with_moved_asset(self):
+        """Loads a project with moved asset."""
+        app = common.create_pitivi(proxyingStrategy=ProxyingStrategy.NOTHING)
+
+        proj_uri = self.create_project_file_from_xges(app, """<ges version='0.3'>
+            <project properties='properties;' metadatas='metadatas;'>
+                <ressources>
+                    <asset id='file://this/is/a/moved/asset.mp4' extractable-type-name='GESUriClip'
+                        properties='properties, supported-formats=(int)6, duration=(guint64)1228000000;' metadatas='metadatas' />
+                </ressources>
+            </project>
+            </ges>""")
+        project_manager = app.project_manager
+        medialib = medialibrary.MediaLibraryWidget(app)
+
+        mainloop = common.create_main_loop()
+
+        def new_project_loaded_cb(*args, **kwargs):
+            mainloop.quit()
+
+        def missing_uri_cb(project_manager, project, unused_error, asset):
+            return common.get_sample_uri("1sec_simpsons_trailer.mp4")
+
+        project_manager.connect("missing-uri", missing_uri_cb)
+        project_manager.connect("new-project-loaded", new_project_loaded_cb)
+
+        project_manager.loadProject(proj_uri)
+        with common.cloned_sample("1sec_simpsons_trailer.mp4"):
+            mainloop.run()
+        self.assertEqual(medialib._progressbar.get_fraction(), 1.0)
+
+    def test_loading_project_with_moved_assets_and_deleted_proxy(self):
+        """Loads a project with moved asset as deleted proxy file."""
+
+        mainloop = common.create_main_loop()
+
+        def proxy_ready_cb(unused_proxy_manager, asset, proxy):
+            mainloop.quit()
+
+        app = common.create_pitivi(proxyingStrategy=ProxyingStrategy.ALL)
+        app.proxy_manager.connect("proxy-ready", proxy_ready_cb)
+
+        proj_uri = self.create_project_file_from_xges(app, """<ges version='0.3'>
+  <project properties='properties;' metadatas='metadatas, name=(string)&quot;New\ Project&quot;, author=(string)Unknown, render-scale=(double)100, format-version=(string)0.3;'>
+    <ressources>
+      <asset id='file:///nop/1sec_simpsons_trailer.mp4' extractable-type-name='GESUriClip' properties='properties, supported-formats=(int)6, duration=(guint64)1228000000;' metadatas='metadatas, audio-codec=(string)&quot;MPEG-4\ AAC\ audio&quot;, maximum-bitrate=(uint)130625, bitrate=(uint)130625, datetime=(datetime)2007-02-19T05:03:04Z, encoder=(string)Lavf54.6.100, container-format=(string)&quot;ISO\ MP4/M4A&quot;, video-codec=(string)&quot;H.264\ /\ AVC&quot;, file-size=(guint64)232417;'  proxy-id='file:///nop/1sec_simpsons_trailer.mp4.232417.proxy.mkv' />
+      <asset id='file:///nop/tears_of_steel.webm' extractable-type-name='GESUriClip' properties='properties, supported-formats=(int)6, duration=(guint64)2003000000;' metadatas='metadatas, container-format=(string)Matroska, language-code=(string)und, application-name=(string)Lavc56.60.100, encoder=(string)&quot;Xiph.Org\ libVorbis\ I\ 20150105\ \(\342\233\204\342\233\204\342\233\204\342\233\204\)&quot;, encoder-version=(uint)0, audio-codec=(string)Vorbis, nominal-bitrate=(uint)80000, bitrate=(uint)80000, video-codec=(string)&quot;VP8\ video&quot;, file-size=(guint64)223340;' />
+      <asset id='file:///nop/1sec_simpsons_trailer.mp4.232417.proxy.mkv' extractable-type-name='GESUriClip' properties='properties, supported-formats=(int)6, duration=(guint64)1228020833;' metadatas='metadatas, container-format=(string)Matroska, audio-codec=(string)Opus, language-code=(string)en, encoder=(string)Lavf54.6.100, bitrate=(uint)64000, video-codec=(string)&quot;Motion\ JPEG&quot;, file-size=(guint64)4694708;' />
+      <asset id='file:///nop/tears_of_steel.webm.223340.proxy.mkv' extractable-type-name='GESUriClip' properties='properties, supported-formats=(int)6, duration=(guint64)2003000000;' metadatas='metadatas, container-format=(string)Matroska, language-code=(string)und, application-name=(string)Lavc56.60.100, encoder=(string)&quot;Xiph.Org\ libVorbis\ I\ 20150105\ \(\342\233\204\342\233\204\342\233\204\342\233\204\)&quot;, encoder-version=(uint)0, audio-codec=(string)Vorbis, nominal-bitrate=(uint)80000, bitrate=(uint)80000, video-codec=(string)&quot;VP8\ video&quot;, file-size=(guint64)223340;' />
+    </ressources>
+</project>
+</ges>""")
+        project_manager = app.project_manager
+        medialib = medialibrary.MediaLibraryWidget(app)
+
+        # Remove proxy
+        with common.cloned_sample("1sec_simpsons_trailer.mp4", "tears_of_steel.webm"):
+            def new_project_loaded_cb(*args, **kwargs):
+                mainloop.quit()
+
+            missing_uris = []
+
+            def missing_uri_cb(project_manager, project, unused_error, asset):
+                missing_uris.append(asset.props.id)
+                return common.get_sample_uri(os.path.basename(asset.props.id))
+
+            project_manager.connect("missing-uri", missing_uri_cb)
+            project_manager.connect("new-project-loaded", new_project_loaded_cb)
+
+            project_manager.loadProject(proj_uri)
+            mainloop.run()
+            self.assertEqual(len(missing_uris), 1,
+                "missing_uri_cb should be called only once, got %s." % missing_uris)
+            self.assertEqual(medialib._progressbar.get_fraction(), 1.0)
+            mainloop.run()
+            self.assertEqual(len(medialib.storemodel), 2,
+                "We should have one asset displayed in the MediaLibrary.")
+            self.assertEqual(medialib.storemodel[0][medialibrary.COL_THUMB_DECORATOR].state,
+                             medialibrary.AssetThumbnail.PROXIED)
+            self.assertEqual(medialib.storemodel[1][medialibrary.COL_THUMB_DECORATOR].state,
+                             medialibrary.AssetThumbnail.IN_PROGRESS)
+
 
 class TestProjectSettings(common.TestCase):
 
@@ -490,8 +586,31 @@ class TestProjectSettings(common.TestCase):
         self.assertEqual(1080, project.videoheight)
         project.videorate = Gst.Fraction(50, 7)
         self.assertEqual(Gst.Fraction(50, 7), project.videorate)
-        project.videopar = Gst.Fraction(2, 7)
-        self.assertEqual(Gst.Fraction(2, 7), project.videopar)
+
+    def testSetAudioProp(self):
+        timeline = common.create_timeline_container()
+        project = timeline.app.project_manager.current_project
+        project.addUris([common.get_sample_uri("mp3_sample.mp3")])
+
+        audio_track = [t for t in project.ges_timeline.tracks if isinstance(t, GES.AudioTrack)][0]
+        mainloop = common.create_main_loop()
+
+        def progress_cb(project, progress, estimated_time):
+            if progress == 100:
+                mainloop.quit()
+
+        project.connect_after("asset-loading-progress", progress_cb)
+        mainloop.run()
+
+        expected = Gst.Caps("audio/x-raw,channels=(int)2,rate=(int)44100")
+        ccaps = audio_track.props.restriction_caps
+        self.assertTrue(ccaps.is_equal_fixed(expected), "%s != %s" % (ccaps, expected))
+
+        project.audiochannels = 6
+
+        expected = Gst.Caps("audio/x-raw,channels=(int)6,rate=(int)44100")
+        ccaps = audio_track.props.restriction_caps
+        self.assertTrue(ccaps.is_equal_fixed(expected), "%s != %s" % (ccaps, expected))
 
     def testInitialization(self):
         mainloop = common.create_main_loop()
@@ -534,7 +653,6 @@ class TestProjectSettings(common.TestCase):
         self.assertEqual(960, project.videowidth)
         self.assertEqual(400, project.videoheight)
         self.assertEqual(Gst.Fraction(24, 1), project.videorate)
-        self.assertEqual(Gst.Fraction(1, 1), project.videopar)
 
     def testLoad(self):
         project = Project(uri="fake.xges", app=common.create_pitivi_mock())
@@ -542,15 +660,15 @@ class TestProjectSettings(common.TestCase):
         self.assertFalse(project._has_default_audio_settings)
 
 
-class TestExportSettings(TestCase):
+class TestExportSettings(common.TestCase):
 
-    def testMasterAttributes(self):
-        self._testMasterAttribute('muxer', dependant_attr='containersettings')
-        self._testMasterAttribute('vencoder', dependant_attr='vcodecsettings')
-        self._testMasterAttribute('aencoder', dependant_attr='acodecsettings')
+    def test_master_attributes(self):
+        self._check_master_attribute("muxer", dependant_attr="containersettings")
+        self._check_master_attribute("vencoder", dependant_attr="vcodecsettings")
+        self._check_master_attribute("aencoder", dependant_attr="acodecsettings")
 
-    def _testMasterAttribute(self, attr, dependant_attr):
-        """Test changing the specified attr has effect on its dependant attr."""
+    def _check_master_attribute(self, attr, dependant_attr):
+        """Test changing the specified attr has effect on its dependent attr."""
         project = common.create_project()
 
         attr_value1 = "%s_value1" % attr
@@ -583,3 +701,37 @@ class TestExportSettings(TestCase):
         setattr(project, attr, attr_value2)
         self.assertFalse("key1" in getattr(project, dependant_attr))
         self.assertFalse("key2" in getattr(project, dependant_attr))
+
+    def test_set_rendering(self):
+        """Checks the set_rendering method."""
+        mainloop = common.create_main_loop()
+
+        def loaded_cb(project, timeline):
+            project.addUris([common.get_sample_uri("tears_of_steel.webm")])
+
+        def progress_cb(project, progress, estimated_time):
+            if progress == 100:
+                mainloop.quit()
+
+        # Create a blank project and add some assets.
+        project = common.create_project()
+
+        project.connect_after("loaded", loaded_cb)
+        project.connect_after("asset-loading-progress", progress_cb)
+
+        mainloop.run()
+
+        # The video settings should match tears_of_steel.webm
+        self.assertEqual(project.videowidth, 960)
+        self.assertEqual(project.videoheight, 400)
+
+        project.render_scale = 3
+        # Pretend we're rendering.
+        project.set_rendering(True)
+        self.assertEqual(project.videowidth, 28)
+        self.assertEqual(project.videoheight, 12)
+
+        # Pretend we're not rendering anymore.
+        project.set_rendering(False)
+        self.assertEqual(project.videowidth, 960)
+        self.assertEqual(project.videoheight, 400)
