@@ -21,12 +21,10 @@ from unittest import mock
 from gi.repository import GES
 
 from pitivi.timeline.layer import Layer
-from tests.common import create_timeline_container
-from tests.common import get_sample_uri
-from tests.common import TestCase
+from tests import common
 
 
-class TestLayerControl(TestCase):
+class TestLayerControl(common.TestCase):
 
     def test_name(self):
         timeline = mock.MagicMock()
@@ -51,15 +49,15 @@ class TestLayerControl(TestCase):
         self.assertEqual(layer.getName(), "Layer 0x")
 
 
-class TestLayer(TestCase):
+class TestLayer(common.TestCase):
 
     def test_check_media_types_when_no_control_ui(self):
         ges_layer = GES.Layer()
-        png = get_sample_uri("flat_colour1_640x480.png")
+        png = common.get_sample_uri("flat_colour1_640x480.png")
         video_clip = GES.UriClipAsset.request_sync(png).extract()
         self.assertTrue(ges_layer.add_clip(video_clip))
         self.assertEqual(len(ges_layer.get_clips()), 1)
-        timeline_container = create_timeline_container()
+        timeline_container = common.create_timeline_container()
         timeline = timeline_container.timeline
         # This will add widgets for the clips in ges_layer and
         # the layer will use checkMediaTypes which updates the
