@@ -20,8 +20,10 @@ from unittest import mock
 
 from gi.repository import Gdk
 from gi.repository import GES
+from gi.repository import Gst
 from gi.repository import Gtk
 
+from pitivi.utils.timeline import UNSELECT
 from pitivi.utils.ui import LAYER_HEIGHT
 from pitivi.utils.ui import SEPARATOR_HEIGHT
 from tests import common
@@ -252,6 +254,37 @@ class TestLayers(BaseTestTimeline):
 
 class TestGrouping(BaseTestTimeline):
 
+    def __check_can_group_ungroup(self, timeline_container, can_group, can_ungroup):
+        self.assertEqual(can_group, timeline_container.group_action.props.enabled)
+        self.assertEqual(can_ungroup, timeline_container.ungroup_action.props.enabled)
+
+    def test_can_group_ungroup(self):
+        timeline_container = create_timeline_container()
+        timeline = timeline_container.timeline
+        self.__check_can_group_ungroup(timeline_container, False, False)
+        ges_clip, = self.addClipsSimple(timeline, 1)
+        self.toggle_clip_selection(ges_clip, expect_selected=True)
+        self.__check_can_group_ungroup(timeline_container, False, True)
+
+        timeline_container.ungroup_action.emit("activate", None)
+        self.__check_can_group_ungroup(timeline_container, False, False)
+
+        layer, = timeline.ges_timeline.get_layers()
+        ges_clip0, ges_clip1 = layer.get_clips()
+        self.toggle_clip_selection(ges_clip0, expect_selected=True)
+        self.__check_can_group_ungroup(timeline_container, False, False)
+
+        # Press <ctrl> so selecting in ADD mode
+        event = mock.Mock()
+        event.keyval = Gdk.KEY_Control_L
+        timeline_container.do_key_press_event(event)
+
+        self.toggle_clip_selection(ges_clip1, expect_selected=True)
+        self.__check_can_group_ungroup(timeline_container, True, False)
+
+        timeline_container.group_action.emit("activate", None)
+        self.__check_can_group_ungroup(timeline_container, False, True)
+
     def group_clips(self, timeline_container, clips):
         timeline = timeline_container.timeline
         timeline.app.settings.leftClickAlsoSeeks = False
@@ -260,6 +293,8 @@ class TestGrouping(BaseTestTimeline):
         event = mock.Mock()
         event.keyval = Gdk.KEY_Control_L
         timeline_container.do_key_press_event(event)
+        timeline.get_clicked_layer_and_pos = mock.Mock()
+        timeline.get_clicked_layer_and_pos.return_value = (None, None)
 
         # Select the 2 clips
         for clip in clips:
@@ -354,6 +389,8 @@ class TestGrouping(BaseTestTimeline):
         event = mock.Mock()
         event.keyval = Gdk.KEY_Control_L
         timeline_container.do_key_press_event(event)
+        timeline.get_clicked_layer_and_pos = mock.Mock()
+        timeline.get_clicked_layer_and_pos.return_value = (None, None)
         self.toggle_clip_selection(clips[1], expect_selected=True)
         timeline_container.do_key_release_event(event)
 
@@ -449,6 +486,8 @@ class TestCopyPaste(BaseTestTimeline):
         event = mock.Mock()
         event.keyval = Gdk.KEY_Control_L
         timeline_container.do_key_press_event(event)
+        timeline.get_clicked_layer_and_pos = mock.Mock()
+        timeline.get_clicked_layer_and_pos.return_value = (None, None)
 
         # Select the 2 clips
         for clip in clips:
@@ -462,19 +501,17 @@ class TestCopyPaste(BaseTestTimeline):
         return timeline_container
 
     def testCopyPaste(self):
-        position = 20
-
         timeline_container = self.copyClips(2)
         timeline = timeline_container.timeline
-
         layer = timeline.ges_timeline.get_layers()[0]
-        # Monkey patching the pipeline.getPosition method
         project = timeline.ges_timeline.get_asset()
-        project.pipeline.getPosition = mock.Mock(return_value=position)
 
         clips = layer.get_clips()
         self.assertEqual(len(clips), 2)
 
+        # Pasting clips for the first time.
+        position = 20
+        project.pipeline.getPosition = mock.Mock(return_value=position)
         timeline_container.paste_action.emit("activate", None)
 
         n_clips = layer.get_clips()
@@ -485,6 +522,19 @@ class TestCopyPaste(BaseTestTimeline):
         self.assertEqual(copied_clips[0].props.start, position)
         self.assertEqual(copied_clips[1].props.start, position + 10)
 
+        # Pasting same clips second time.
+        position = 40
+        project.pipeline.getPosition = mock.Mock(return_value=position)
+        timeline_container.paste_action.emit("activate", None)
+
+        n_clips = layer.get_clips()
+        self.assertEqual(len(n_clips), 6)
+
+        copied_clips = [clip for clip in n_clips if clip not in clips]
+        self.assertEqual(len(copied_clips), 4)
+        self.assertEqual(copied_clips[2].props.start, position)
+        self.assertEqual(copied_clips[3].props.start, position + 10)
+
 
 class TestEditing(BaseTestTimeline):
 
@@ -519,3 +569,174 @@ class TestEditing(BaseTestTimeline):
         timeline._button_release_event_cb(None, event)
         self.assertEqual(len(timeline.ges_timeline.get_layers()), 1,
                          "No new layer should have been created")
+
+
+class TestShiftSelection(BaseTestTimeline):
+
+    def __reset_clips_selection(self, timeline):
+        """Unselects all clips in the timeline."""
+        layers = timeline.ges_timeline.get_layers()
+        for layer in layers:
+            clips = layer.get_clips()
+            timeline.selection.setSelection(clips, UNSELECT)
+            timeline.set_selection_meta_info(layer, 0, UNSELECT)
+
+    def __check_selected(self, selected_clips, not_selected_clips):
+        for clip in selected_clips:
+            self.assertEqual(clip.selected._selected, True)
+        for clip in not_selected_clips:
+            self.assertEqual(clip.selected._selected, False)
+
+    def __check_simple(self, left_click_also_seeks):
+        timeline_container = create_timeline_container()
+        timeline = timeline_container.timeline
+        timeline.app.settings.leftClickAlsoSeeks = left_click_also_seeks
+        ges_layer = timeline.ges_timeline.append_layer()
+        asset = GES.UriClipAsset.request_sync(
+            common.get_sample_uri("1sec_simpsons_trailer.mp4"))
+        ges_clip1 = ges_layer.add_asset(asset, 0 * Gst.SECOND, 0,
+            1 * Gst.SECOND, GES.TrackType.UNKNOWN)
+        ges_clip2 = ges_layer.add_asset(asset, 1 * Gst.SECOND, 0,
+            1 * Gst.SECOND, GES.TrackType.UNKNOWN)
+
+        event = mock.Mock()
+        event.get_button.return_value = (True, 1)
+        timeline._seek = mock.Mock()
+        timeline._seek.return_value = True
+        timeline.get_clicked_layer_and_pos = mock.Mock()
+
+        with mock.patch.object(Gtk, "get_event_widget") as get_event_widget:
+            get_event_widget.return_value = timeline
+
+            # Simulate click on first and shift+click on second clip.
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer, 0.5 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            timeline.get_parent()._shiftMask = True
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer, 1.5 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            self.__check_selected([ges_clip1, ges_clip2], [])
+
+    def test_simple(self):
+        self.__check_simple(left_click_also_seeks=False)
+        self.__check_simple(left_click_also_seeks=True)
+
+    def __check_shift_selection_single_layer(self, left_click_also_seeks):
+        """Checks group clips selection across a single layer."""
+        timeline_container = create_timeline_container()
+        timeline = timeline_container.timeline
+        timeline.app.settings.leftClickAlsoSeeks = left_click_also_seeks
+        ges_layer = timeline.ges_timeline.append_layer()
+        ges_clip1 = self.add_clip(ges_layer, 5 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_clip2 = self.add_clip(ges_layer, 15 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_clip3 = self.add_clip(ges_layer, 25 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_clip4 = self.add_clip(ges_layer, 35 * Gst.SECOND, duration=2 * Gst.SECOND)
+
+        event = mock.Mock()
+        event.get_button.return_value = (True, 1)
+        timeline.get_parent()._shiftMask = True
+        timeline._seek = mock.Mock()
+        timeline._seek.return_value = True
+        timeline.get_clicked_layer_and_pos = mock.Mock()
+
+        with mock.patch.object(Gtk, "get_event_widget") as get_event_widget:
+            get_event_widget.return_value = timeline
+
+            # Simulate shift+click before first and on second clip.
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer, 1 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer, 17 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            self.__check_selected([ges_clip1, ges_clip2], [ges_clip3, ges_clip4])
+            self.__reset_clips_selection(timeline)
+            timeline.resetSelectionGroup()
+
+            # Simiulate shift+click before first and after fourth clip.
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer, 1 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer, 39 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            self.__check_selected([ges_clip1, ges_clip2, ges_clip3, ges_clip4], [])
+            self.__reset_clips_selection(timeline)
+            timeline.resetSelectionGroup()
+
+            # Simiulate shift+click on first, after fourth and before third clip.
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer, 6 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer, 40 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer, 23 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            self.__check_selected([ges_clip1, ges_clip2], [ges_clip3, ges_clip4])
+            self.__reset_clips_selection(timeline)
+            timeline.resetSelectionGroup()
+
+            # Simulate shift+click twice on the same clip.
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer, 6 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer, 6.5 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            self.__check_selected([ges_clip1], [ges_clip2, ges_clip3, ges_clip4])
+
+    def test_shift_selection_single_layer(self):
+        self.__check_shift_selection_single_layer(left_click_also_seeks=False)
+        self.__check_shift_selection_single_layer(left_click_also_seeks=True)
+
+    def __check_shift_selection_multiple_layers(self, left_click_also_seeks):
+        """Checks group clips selection across multiple layers."""
+        timeline_container = create_timeline_container()
+        timeline = timeline_container.timeline
+        timeline.app.settings.leftClickAlsoSeeks = left_click_also_seeks
+        ges_layer1 = timeline.ges_timeline.append_layer()
+        ges_clip11 = self.add_clip(ges_layer1, 5 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_clip12 = self.add_clip(ges_layer1, 15 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_clip13 = self.add_clip(ges_layer1, 25 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_layer2 = timeline.ges_timeline.append_layer()
+        ges_clip21 = self.add_clip(ges_layer2, 0 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_clip22 = self.add_clip(ges_layer2, 6 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_clip23 = self.add_clip(ges_layer2, 21 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_layer3 = timeline.ges_timeline.append_layer()
+        ges_clip31 = self.add_clip(ges_layer3, 3 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_clip32 = self.add_clip(ges_layer3, 10 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_clip33 = self.add_clip(ges_layer3, 18 * Gst.SECOND, duration=2 * Gst.SECOND)
+
+        event = mock.Mock()
+        event.get_button.return_value = (True, 1)
+        timeline.get_parent()._shiftMask = True
+        timeline._seek = mock.Mock()
+        timeline._seek.return_value = True
+        timeline.get_clicked_layer_and_pos = mock.Mock()
+
+        with mock.patch.object(Gtk, "get_event_widget") as get_event_widget:
+            get_event_widget.return_value = timeline
+
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer2, 3 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer1, 9 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            self.__check_selected([ges_clip11, ges_clip22], [ges_clip12, ges_clip13,
+                ges_clip21, ges_clip23, ges_clip31, ges_clip32, ges_clip33])
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer3, 12 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            self.__check_selected([ges_clip22, ges_clip31, ges_clip32], [ges_clip11,
+                ges_clip12, ges_clip13, ges_clip21, ges_clip23, ges_clip33])
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer1, 22 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            self.__check_selected([ges_clip11, ges_clip12, ges_clip22, ges_clip23],
+                [ges_clip13, ges_clip21, ges_clip31, ges_clip32, ges_clip33])
+            self.__reset_clips_selection(timeline)
+            timeline.resetSelectionGroup()
+
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer1, 3 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer2, 26 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            self.__check_selected([ges_clip11, ges_clip12, ges_clip13, ges_clip22, ges_clip23],
+                [ges_clip21, ges_clip31, ges_clip32, ges_clip33])
+            timeline.get_clicked_layer_and_pos.return_value = (ges_layer3, 30 * Gst.SECOND)
+            timeline._button_release_event_cb(None, event)
+            self.__check_selected([ges_clip11, ges_clip12, ges_clip13, ges_clip22, ges_clip23,
+                ges_clip31, ges_clip32, ges_clip33], [ges_clip21])
+
+    def test_shift_selection_multiple_layers(self):
+        self.__check_shift_selection_multiple_layers(left_click_also_seeks=False)
+        self.__check_shift_selection_multiple_layers(left_click_also_seeks=True)
