@@ -18,8 +18,6 @@
 # Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 # Boston, MA 02110-1301, USA.
 from unittest import mock
-from unittest import skip
-from unittest import TestCase
 
 from gi.repository import Gdk
 from gi.repository import GES
@@ -40,9 +38,10 @@ from pitivi.utils.ui import URI_TARGET_ENTRY
 from tests import common
 
 
-class BaseTestUndoTimeline(TestCase):
+class BaseTestUndoTimeline(common.TestCase):
 
     def setUp(self):
+        super(BaseTestUndoTimeline, self).setUp()
         self.app = common.create_pitivi()
         self.app.project_manager.newBlankProject()
 
@@ -190,9 +189,7 @@ class TestTimelineObserver(BaseTestUndoTimeline):
         self.assertEqual(len(clips), 1, clips)
         self.assertEqual(len(clips[0].get_children(False)), 2)
 
-        timeline.selection.select([clip])
-        timeline.resetSelectionGroup()
-        timeline.current_group.add(clip)
+        common.TestCase.toggle_clip_selection(self, clips[0], True)
         self.timeline_container.ungroup_action.activate(None)
         clips = list(self.getTimelineClips())
         self.assertEqual(len(clips), 2, clips)
@@ -736,6 +733,39 @@ class TestControlSourceObserver(BaseTestUndoTimeline):
         self.assertEqual(0.9, control_source.get_all()[0].value)
 
 
+class TestTrackElementObserver(BaseTestUndoTimeline):
+
+    def test_effects_index(self):
+        stacks = []
+        self.action_log.connect("commit", BaseTestUndoTimeline.commit_cb, stacks)
+
+        clip1 = GES.TitleClip()
+        self.layer.add_clip(clip1)
+
+        effect1 = GES.Effect.new("agingtv")
+        effect2 = GES.Effect.new("edgetv")
+        clip1.add(effect1)
+        clip1.add(effect2)
+        self.assertEqual(clip1.get_top_effect_index(effect1), 0)
+        self.assertEqual(clip1.get_top_effect_index(effect2), 1)
+
+        with self.action_log.started("move effect"):
+            assert clip1.set_top_effect_index(effect2, 0)
+
+        self.assertEqual(len(stacks), 1)
+
+        self.assertEqual(clip1.get_top_effect_index(effect1), 1)
+        self.assertEqual(clip1.get_top_effect_index(effect2), 0)
+
+        self.action_log.undo()
+        self.assertEqual(clip1.get_top_effect_index(effect1), 0)
+        self.assertEqual(clip1.get_top_effect_index(effect2), 1)
+
+        self.action_log.redo()
+        self.assertEqual(clip1.get_top_effect_index(effect1), 1)
+        self.assertEqual(clip1.get_top_effect_index(effect2), 0)
+
+
 class TestTimelineElementObserver(BaseTestUndoTimeline):
 
     def testTrackElementPropertyChanged(self):
